@@ -134,6 +134,11 @@ void split_free( char **split, int length );
  */
 char *xstrsep(char **stringp, const char *delim);
 
+/**
+ * string allocation
+ */
+char *stralloc(char *); 
+
 #endif  /* JOEDOG_H */  
 
 
