@@ -1,8 +1,8 @@
 /**
  * Configuration file support
  *
- * Copyright (C) 2000, 2001, 2002 by
- * Jeffrey Fulmer - <jdfulmer@armstrong.com>
+ * Copyright (C) 2000, 2001, 2002, 2005 by
+ * Jeffrey Fulmer - <jeff@joedog.org>
  * This file is distributed as part of Siege 
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,6 +27,8 @@
 #include <joedog/boolean.h>
 #include <joedog/defs.h>
 
+BOOLEAN is_variable_line(char *line);
+
 /** 
  * Ignores comment lines beginning with 
  * '#' empty lines beginning with \n 
@@ -48,7 +50,7 @@ parse(char *str)
  * parse to ignore comments and empty lines. 
  */ 
 int 
-read_cfg_file( LINES *l, char *filename )
+read_cfg_file(LINES *l, char *filename)
 {
   /* file pointer  */
   FILE *file; 
@@ -61,7 +63,7 @@ read_cfg_file( LINES *l, char *filename )
   
   /* make sure LINES has been initialized. */
   if(!l){	
-    printf( "Structure not initialized!\n" );
+    printf("Structure not initialized!\n");
     return -1;
   }
 
@@ -91,7 +93,7 @@ read_cfg_file( LINES *l, char *filename )
     parse(line);
     chomp(line);
     if(strlen(line) == 0);
-    else if(strstr(line, "=")&&!strstr(line, "POST")&&!strstr(line, "?")&&!strstr(line, ";")){
+    else if(is_variable_line(line)){
       char *tmp = line;
       option = tmp;
       while(*tmp && !ISSPACE((int)*tmp) && !ISSEPARATOR(*tmp))
@@ -115,6 +117,7 @@ read_cfg_file( LINES *l, char *filename )
     }
     memset(line, 0, sizeof(line));
   }
+
   fclose(file);
   xfree(line);
   hash_destroy(H);
@@ -122,24 +125,24 @@ read_cfg_file( LINES *l, char *filename )
 }
 
 int
-read_cmd_line( LINES *l, char *url )
+read_cmd_line(LINES *l, char *url)
 {
   int x = 0;
   /* char array to hold contents */
   char head[BUFSIZE];
  
   /* make sure config_t has been initialized. */
-  if( !l ){
-    printf( "Structure not initialized!\n" );
+  if(!l){
+    printf("Structure not initialized!\n");
     return -1;
   }
  
   l->index = 0;
-  while( x < 4 ){
-  snprintf( head, sizeof( head ), "%s", url );
-  parse( head );
-  chomp( head );
-  if( strlen( head ) == 0 );
+  while(x < 4){
+  snprintf(head, sizeof(head), "%s", url);
+  parse(head);
+  chomp(head);
+  if(strlen(head) == 0);
   else{
     l->line = (char**)realloc(l->line, sizeof(char *) * (l->index + 1));
     l->line[l->index] = (char *)strdup( head );
@@ -150,3 +153,34 @@ read_cmd_line( LINES *l, char *url )
   return l->index;
 } 
 
+BOOLEAN
+is_variable_line(char *line)
+{
+  int pos;
+  int x;
+  char c;
+
+  /**
+   * check for variable assignment; make sure that on the left side 
+   * of the = is nothing but letters, numbers, and/or underscores.
+   */
+  pos = (int)strstr(line, "=");
+  if(pos > 0){
+    for(x = 0; x < (pos - (int)line); x++){
+      c = line[x];
+      /* c must be A-Z, a-z, 0-9, or underscore. */
+      if ((c < 'a' || c > 'z') &&
+          (c < 'A' || c > 'Z') &&
+          (c < '0' || c > '9') &&
+          (c != '_')){
+            return FALSE;
+      }
+    }
+  } else {
+    /** 
+     * if it has no '=' then it can't be a variable line 
+     */
+    return FALSE;
+  }
+  return TRUE;
+}
