@@ -1,8 +1,8 @@
 /**
  * HTTP/HTTPS client support
  *
- * Copyright (C) 2000, 2001, 2002, 2003 by
- * Jeffrey Fulmer - <jdfulmer@armstrong.com>
+ * Copyright (C) 2000-2005 by
+ * Jeffrey Fulmer - <jeff@joedog.org>
  * This file is distributed as part of Siege 
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,32 +31,33 @@
 #include <auth.h>
 #include <cookie.h>
 #include <joedog/boolean.h>
-/**
- * see below at "start_routine( CLIENT *c )" for more info on that:
- */
+#include <joedog/defs.h>
+
 #if defined(hpux) || defined(__hpux) || defined(WINDOWS)
 #define SIGNAL_CLIENT_PLATFORM
 #endif
 
 /**
  * local prototypes
- */ 
-int http_request( CONN *C, URL *U, CLIENT *c ); 
-#ifdef SIGNAL_CLIENT_PLATFORM
+ */
+private BOOLEAN http_request(CONN *C, URL *U, CLIENT *c);
+private void increment_failures();
+
+#ifdef  SIGNAL_CLIENT_PLATFORM
 static void signal_handler( int i );
 static void signal_init();
-#else/*CANCEL_CLIENT_PLATFORM*/
-void clean_up(); 
+#else /*CANCEL_CLIENT_PLATFORM*/
+void clean_up();
 #endif/*SIGNAL_CLIENT_PLATFORM*/
-
+ 
 /**
  * local variables
  */
 #ifdef SIGNAL_CLIENT_PLATFORM
-static pthread_once_t once = PTHREAD_ONCE_INIT; 
+static pthread_once_t once = PTHREAD_ONCE_INIT;
 #endif/*SIGNAL_CLIENT_PLATFORM*/
 float highmark = 0;
-float lowmark = -1;
+float lowmark = -1;  
 
 /**
  * The thread entry point for clients.
@@ -77,57 +78,43 @@ start_routine(CLIENT *client)
   int ret;                    /* function return value     */
   CONN *C = NULL;             /* connection data (sock.h)  */ 
 #ifdef SIGNAL_CLIENT_PLATFORM
-  sigset_t  sigs;             /* signal set for sig client */
+  sigset_t  sigs;
 #else
-  int type, state;            /* old cancel type & state   */
+  int type, state;
 #endif 
 
   C = xcalloc(sizeof(CONN), 1);
+  C->sock = -1;
+
 #ifdef SIGNAL_CLIENT_PLATFORM
   pthread_once(&once, signal_init);
   sigemptyset(&sigs);
   sigaddset(&sigs, SIGUSR1);
   pthread_sigmask(SIG_UNBLOCK, &sigs, NULL);
-#else/*CANCEL_CLIENT_PLATFORM*/
+#else
   #if defined(_AIX)
-    pthread_cleanup_push( (void(*)(void*))clean_up, NULL ); 
+    pthread_cleanup_push((void(*)(void*))clean_up, NULL);
   #else
-    pthread_cleanup_push( (void*)clean_up, C ); 
+    pthread_cleanup_push((void*)clean_up, C);
   #endif
-#endif/*SIGNAL_CLIENT_PLATFORM*/
-
-  /** 
-   * lock and increment the thread counter 
-   */
-  pthread_mutex_lock( &our.mutex_lock );
-  if( our.shutting_down != TRUE ){
-    our.total_clients ++;
-    our.total_threads ++;
-  }
-  pthread_mutex_unlock(&our.mutex_lock); 
-  pthread_cond_signal(&our.exit_cond);
-
+#endif /*SIGNAL_CLIENT_PLATFORM*/
 
 #ifdef SIGNAL_CLIENT_PLATFORM
 #else/*CANCEL_CLIENT_PLATFORM*/
-  /**
-   * set the cancel type and cancel state 
-   * by OS type: big issue async vs defer
-   */
-#if defined(sun)
-  pthread_setcanceltype ( PTHREAD_CANCEL_DEFERRED, &type );
-#elif defined(_AIX)
-  pthread_setcanceltype ( PTHREAD_CANCEL_DEFERRED, &type );
-#elif defined(hpux) || defined(__hpux)
-  pthread_setcanceltype ( PTHREAD_CANCEL_ASYNCHRONOUS, &type );
-#else
-  pthread_setcanceltype ( PTHREAD_CANCEL_ASYNCHRONOUS, &type );
-#endif
-  pthread_setcancelstate( PTHREAD_CANCEL_ENABLE, &state ); 
-#endif/*SIGNAL_CLIENT_PLATFORM*/
+  #if defined(sun)
+    pthread_setcanceltype (PTHREAD_CANCEL_DEFERRED, &type);
+  #elif defined(_AIX)
+    pthread_setcanceltype (PTHREAD_CANCEL_DEFERRED, &type);
+  #elif defined(hpux) || defined(__hpux)
+    pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, &type);
+  #else
+    pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, &type);
+  #endif
+    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &state);
+#endif/*SIGNAL_CLIENT_PLATFORM*/ 
 
   if( my.login == TRUE ){
-    http_request( C, add_url( my.loginurl ), client );
+    http_request(C, add_url(my.loginurl, 0), client);
   }
 
   for( x = 0, y = 0; x < my.reps; x++, y++ ){
@@ -137,8 +124,7 @@ start_routine(CLIENT *client)
                           ((double)RAND_MAX + 1) * my.length ) + .5); 
       y = (y >= my.length)?my.length-1:y;
       y = (y < 0)?0:y;
-    }
-    else{
+    } else {
       /**
        * URLs accessed sequentially; when reaching the end, start over
        * with clean slate, ie. reset (delete) cookies (eg. to let a new
@@ -151,38 +137,36 @@ start_routine(CLIENT *client)
         }
       }
     }
-    if(y >= my.length || y < 0){ printf("y out of bounds: %d", y); y = 0; }
-    if(client->U[y]->hostname != NULL)
-      ret = http_request(C, client->U[y], client);
-    else
-      ret = -1;
-    switch(ret){
-    case -1: 
-      my.failed++;
-      break;
-    case  0:
-    default:
-      break;
-    } 
+    if(y >= my.length || y < 0){ 
+      printf("y out of bounds: %d >= %d", y, my.length); 
+      y = 0; 
+    }
+
+    if(client->U[y]->hostname != NULL){
+      if((ret = http_request(C, client->U[y], client))==FALSE){
+        increment_failures();
+      }
+    }
  
-    if(my.failed >= my.failures){
+    if(my.failures > 0 && my.failed >= my.failures){
       break;
     }
   }
 
-#ifdef SIGNAL_CLIENT_PLATFORM
-#else/*CANCEL_CLIENT_PLATFORM*/
+  #ifdef SIGNAL_CLIENT_PLATFORM
+  #else/*CANCEL_CLIENT_PLATFORM*/
   /**
    * every cleanup must have a pop
    */
   pthread_cleanup_pop(0);
-#endif/*SIGNAL_CLIENT_PLATFORM*/
+  #endif/*SIGNAL_CLIENT_PLATFORM*/ 
   if(C->sock >= 0){
     C->connection.reuse = 0;    
     socket_close(C);
   }
   xfree(C);
   C = NULL;
+
   return(NULL);
 }
 
@@ -191,15 +175,35 @@ start_routine(CLIENT *client)
  * above. This invokes the HTTP logic and compiles the
  * statistics for the run.
  */
-int
+private BOOLEAN
 http_request(CONN *C, URL *U, CLIENT *client)
 {
-  int okay = 0;               /* connection check        */
-  int bytes= 0, code, fail;   /* statistics quantifiers  */
-  float etime;                /* trans. elapsed time     */
-  clock_t start, stop;        /* time structs for elaps. */
-  struct tms t_start, t_stop; /* time structs for elaps. */
-  HEADERS *head;              /* HTTP header structure   */ 
+  unsigned long bytes  = 0;
+  int      code, fail;  
+  float    etime; 
+  clock_t  start, stop;
+  struct   tms t_start, t_stop; 
+  HEADERS  *head; 
+#ifdef  HAVE_LOCALTIME_R
+  struct   tm keepsake;
+#endif/*HAVE_LOCALTIME_R*/
+  time_t   now; 
+  struct   tm *tmp;
+  size_t   len;
+  char     fmtime[65];
+
+  if(my.csv){
+    now = time(NULL);
+#ifdef HAVE_LOCALTIME_R
+    tmp = (struct tm *)localtime_r(&now, &keepsake);
+#else
+    tmp = localtime(&now);
+#endif/*HAVE_LOCALTIME_R*/
+    if(tmp)
+      len = strftime(fmtime, 64, "%Y-%m-%d %H:%M:%S", tmp);
+    else
+      snprintf(fmtime, 64, "n/a");
+  }
 
   C->pos_ini              = 0;
   C->inbuffer             = 0; 
@@ -235,34 +239,27 @@ http_request(CONN *C, URL *U, CLIENT *client)
   /* record transaction start time */
   start = times( &t_start );  
 
-  if( my.debug ){ 
-    printf( 
-      "attempting connection to %s:%d\n", 
-      (my.proxy.required==TRUE)?my.proxy.hostname:U->hostname,
-      (my.proxy.required==TRUE)?my.proxy.port:U->port 
-    ); 
-    fflush( stdout ); 
-  } 
+  debug( 
+    "attempting connection to %s:%d\n", 
+    (my.proxy.required==TRUE)?my.proxy.hostname:U->hostname,
+    (my.proxy.required==TRUE)?my.proxy.port:U->port 
+  ); 
 
   if(!C->connection.reuse || C->connection.status == 0){
-    C->sock = new_socket( C, U->hostname, U->port );
+    C->sock = new_socket(C, U->hostname, U->port);
   }
   if( my.keepalive )
     C->connection.reuse = TRUE;
 
   if(C->sock < 0){
-    if( my.debug ){ 
-      printf( "connection failed. erro %d(%s)\n",errno,strerror(errno) ); 
-      fflush( stdout ); 
-    } 
-    okay = 0;
-    fail = 1;
+    debug(
+      "connection failed. erro %d(%s)",errno,strerror(errno)
+    ); 
     socket_close(C);
-  }  
-  else{ okay = 1; }
-  if(!okay){ return -1; }
+    return FALSE;
+  } 
 
-  if(my.debug){ printf( "connention made.\n" ); fflush(stdout); } 
+  debug("client.c:%d connection made: %d", __LINE__, C->sock); 
 
   if(C->prot == HTTPS){
     if(my.proxy.required){
@@ -270,37 +267,39 @@ http_request(CONN *C, URL *U, CLIENT *client)
       https_tunnel_response(C);
     }
     C->encrypt = TRUE;
-    SSL_initialize(C);
+    if(SSL_initialize(C)==FALSE){
+      return FALSE;
+    }
   }
 
   /**
    * write to socket with a POST or GET
    */
-  if( U->calltype == URL_POST ){ 
+  if(U->calltype == URL_POST){ 
     if((http_post(C, U->hostname, U->port, U->pathname, U->postdata, U->postlen)) < 0){
       C->connection.reuse = 0;
       socket_close(C);
-      return -1;
+      return FALSE;
     }
-  }
-  else{ 
+  } else { 
     if((http_get(C, U->hostname, U->port, U->pathname)) < 0){
       C->connection.reuse = 0;
-      socket_close( C );
-      return -1;
+      socket_close(C);
+      return FALSE;
     }
   } 
   /**
    * read from socket and collect statistics.
    */
   head     =  http_read_headers(C, U->hostname); 
-  if(!head){  C->connection.reuse = 0; socket_close(C); return -1; } 
+  if(!head){  C->connection.reuse = 0; socket_close(C); return FALSE; } 
   bytes    =  http_read(C); 
   if( !my.zero_ok && (bytes < 1) ){ 
     C->connection.reuse = 0; 
     socket_close(C); 
     xfree(head); 
-    return -1; 
+    puts("zero bytes");
+    return FALSE; 
   } 
   stop     =  times(&t_stop); 
   etime    =  elapsed_time(stop - start);  
@@ -324,20 +323,34 @@ http_request(CONN *C, URL *U, CLIENT *client)
   }
   client->bigtime = highmark;
   client->smalltime = lowmark;
+
   /**
    * verbose output, print statistics to stdout
    */
-  if((my.verbose && !my.get)&&( our.shutting_down != TRUE )&&( !my.debug )){
-    if( my.display )
-      printf(
-        "%4d: %s %d %6.2f secs: %7d bytes ==> %s\n", client->id,
-        head->head, head->code, etime, bytes, U->pathname
-      ); 
-    else
-      printf( 
-        "%s %d %6.2f secs: %7d bytes ==> %s\n", 
-        head->head, head->code, etime, bytes, U->pathname
+  if((my.verbose && !my.get) && (!my.debug)){
+    if(my.csv){
+      if(my.display)
+        printf("%s%s%4d,%s,%d,%6.2f,%7lu,%s,%d,%s\n",
+        (my.mark)?my.markstr:"", (my.mark)?",":"", client->id, head->head, head->code, 
+        etime, bytes, (my.fullurl)?U->url:U->pathname, U->urlid, fmtime
       );
+      else
+        printf("%s%s%s,%d,%6.2f,%7lu,%s,%d,%s\n",
+          (my.mark)?my.markstr:"", (my.mark)?",":"", head->head, head->code, 
+          etime, bytes, (my.fullurl)?U->url:U->pathname, U->urlid, fmtime
+        );
+    } else {
+      if(my.display)
+        printf(
+          "%4d: %s %d %6.2f secs: %7lu bytes ==> %s\n", client->id,
+          head->head, head->code, etime, bytes, (my.fullurl)?U->url:U->pathname
+        ); 
+      else
+        printf( 
+          "%s %d %6.2f secs: %7lu bytes ==> %s\n", 
+          head->head, head->code, etime, bytes, (my.fullurl)?U->url:U->pathname
+        );
+    } /* else not my.csv */
   }
 
   /**
@@ -354,15 +367,15 @@ http_request(CONN *C, URL *U, CLIENT *client)
   case 302:
     redirect_url = (URL*)xmalloc(sizeof(URL));
     if(my.follow && head->redirect[0]){
-      if(my.debug){ printf("parsing redirection URL %s\n", head->redirect); }
+      debug("parsing redirection URL %s", head->redirect);
       if(protocol_length(head->redirect) == 0){
           memcpy(redirect_url, U, sizeof(URL));
           redirect_url->pathname = head->redirect;
       } else {
-        redirect_url = add_redirect(head->redirect, U->port);
+        redirect_url = add_redirect(head->redirect, U->port, U->urlid);
       }
-      if(( http_request( C, redirect_url, client )) < 0 )
-        return -1;
+      if((http_request(C, redirect_url, client)) == FALSE)
+        return FALSE;
     }
     xfree(redirect_url);
     break;
@@ -380,9 +393,9 @@ http_request(CONN *C, URL *U, CLIENT *client)
         client->auth.type.www =  BASIC;
         set_authorization(WWW, head->auth.realm.www);
       }
-      if((http_request(C, U, client)) < 0){
+      if((http_request(C, U, client)) == FALSE){
         fprintf(stderr, "ERROR from http_request\n");
-        return -1;
+        return FALSE;
       }
     }
     break;
@@ -400,8 +413,8 @@ http_request(CONN *C, URL *U, CLIENT *client)
         client->auth.type.proxy =  BASIC;
         set_authorization(PROXY, head->auth.realm.proxy);
       }
-      if((http_request(C, U, client)) < 0)
-        return -1;
+      if((http_request(C, U, client)) == FALSE)
+        return FALSE;
     }
     break;
   default:
@@ -412,18 +425,17 @@ http_request(CONN *C, URL *U, CLIENT *client)
   xfree(head->redirect);
   xfree(head); 
 
-  /* NOTE: Join all static read threads */
-  return 0;
+  return TRUE;
 }
 
 #ifdef SIGNAL_CLIENT_PLATFORM
-static void
-signal_handler( int sig )
+private void
+signal_handler(int sig)
 {
-  pthread_exit( &sig );
+  pthread_exit(&sig);
 }
  
-static void
+private void
 signal_init()
 {
   struct sigaction sa;
@@ -434,17 +446,19 @@ signal_init()
   sigaction(SIGUSR1, &sa, NULL);
 }
 #else/*CANCEL_CLIENT_PLATFORM*/
-/**
- * called by pthread_cleanup and invoked
- * on pthread_cancel. We de-increment the
- * thread total before exiting.
- */
+
 void
 clean_up()
 {
-  pthread_mutex_lock(&our.mutex_lock);
-  our.total_threads --;
-  pthread_cond_signal(&our.exit_cond);
-  pthread_mutex_unlock(&our.mutex_lock); 
+  return;
 }
 #endif
+
+private void
+increment_failures()
+{
+  pthread_mutex_lock(&(my.lock));  
+  my.failed++;
+  pthread_mutex_unlock(&(my.lock));  
+}
+
