@@ -23,8 +23,8 @@
  *
  */ 
 #include <setup.h>
-#include <cookie.h>
 #include <time.h>
+#include <cookie.h>
 #include <joedog/joedog.h>
 #include <joedog/boolean.h>
 
@@ -47,6 +47,8 @@ free_cookie(PARSED_COOKIE* ck)
   xfree(ck->path);
 }
 
+COOKIE *cookie;
+
 /**  
  * This function takes a string in the format
  * "Mon, 01-Jan-96 13:45:35 GMT" or "Mon,  1 Jan 1996 13:45:35 GMT" 
@@ -57,7 +59,12 @@ mktime2(const char *string, int absolute)
 {
   const char *s;
   time_t now, clock2;
-  int day, month, year, hour, minutes, seconds;
+  int day     = 0;
+  int month   = 0;
+  int year    = 0; 
+  int hour    = 0; 
+  int minutes = 0; 
+  int seconds = 0;
   const char *start;
   char temp[8];
 
@@ -74,7 +81,7 @@ mktime2(const char *string, int absolute)
    */
   while(*s != '\0' && !isdigit((unsigned char)*s))
     s++;
-  if( *s == '\0' )
+  if(*s == '\0')
     return(0);
 
   /*
@@ -83,18 +90,20 @@ mktime2(const char *string, int absolute)
   start = s;
   while( *s != '\0' && isdigit((unsigned char)*s))
     s++;
-  if( *s == '\0' || (s - start) > 2 )
+  if(*s == '\0' || (s - start) > 2){
     return(0);
+  }
   strncpy(temp, start, (int)(s - start));
-  day = atoi(temp);
-  if( day < 1 || day > 31 )
+  if(temp != NULL)
+    day = atoi(temp);
+  if(day < 1 || day > 31)
     return(0);
   /*
    *  Get the month string and convert to an integer.
    */
-  while( *s != '\0' && !isalnum((unsigned char)*s))
+  while(*s != '\0' && !isalnum((unsigned char)*s))
     s++;
-  if( *s == '\0' )
+  if(*s == '\0')
     return(0);
   start = s;
   while( *s != '\0' && isalnum((unsigned char)*s))
@@ -108,34 +117,35 @@ mktime2(const char *string, int absolute)
   switch( toupper(temp[0])){
     case '0':
     case '1':
-      month = atoi(temp);
+      if(temp != NULL)
+        month = atoi(temp);
       if( month < 1 || month > 12 )
         return(0);
       break;
     case 'A':
-      if( !strcasecmp( temp, "Apr" ))
+      if(!strcasecmp(temp, "Apr"))
         month = 4;
-      else if( !strcasecmp( temp, "Aug" ))
+      else if(!strcasecmp(temp, "Aug"))
         month = 8;
       else
         return(0);
       break;
     case 'D':
-      if( !strcasecmp( temp, "Dec" ))
+      if(!strcasecmp(temp, "Dec"))
         month = 12;
       else
         return(0);
       break;
     case 'F':
-      if( !strcasecmp( temp, "Feb" )) 
+      if(!strcasecmp(temp, "Feb")) 
         month = 2;
       else
         return(0);
       break;
     case 'J':
-      if( !strcasecmp( temp, "Jan" )) 
+      if(!strcasecmp(temp, "Jan")) 
         month = 1;
-      else if( !strcasecmp( temp, "Jun" )) 
+      else if(!strcasecmp(temp, "Jun")) 
         month = 6;
       else if (!strcasecmp(temp, "Jul")) 
         month = 7;
@@ -143,27 +153,27 @@ mktime2(const char *string, int absolute)
         return(0);
       break;
     case 'M':
-      if( !strcasecmp( temp, "Mar" )) 
+      if(!strcasecmp(temp, "Mar")) 
         month = 3;
-      else if( !strcasecmp( temp, "May" )) 
+      else if(!strcasecmp(temp, "May")) 
         month = 5;
       else 
         return(0);
       break;
     case 'N':
-      if( !strcasecmp( temp, "Nov" ))
+      if(!strcasecmp(temp, "Nov"))
         month = 11;
       else 
         return(0);
       break;
     case 'O':
-      if( !strcasecmp( temp, "Oct" )) 
+      if(!strcasecmp(temp, "Oct")) 
         month = 10;
       else 
         return(0);
       break;
     case 'S':
-      if( !strcasecmp( temp, "Sep" ))
+      if(!strcasecmp(temp, "Sep"))
         month = 9;
       else 
         return(0);
@@ -175,17 +185,16 @@ mktime2(const char *string, int absolute)
   /*
    *  Get the numeric year string and convert to an integer.
    */
-  while( *s != '\0' && !isdigit(( unsigned char)*s ))
+  while(*s != '\0' && !isdigit((unsigned char)*s))
     s++;
-  if( *s == '\0' )
+  if(*s == '\0')
     return(0);
   start = s;
-  while( *s != '\0' && isdigit(( unsigned char)*s ))
+  while(*s != '\0' && isdigit((unsigned char)*s))
     s++;
-  if(( s - start ) == 4 ){
+  if((s - start) == 4){
     strncpy(temp, start, 4);
-  } 
-  else if(( s - start ) == 2 ){
+  } else if((s - start) == 2){
     now = time(NULL);
     /*
      * Assume that received 2-digit dates >= 70 are 19xx; others
@@ -196,69 +205,71 @@ mktime2(const char *string, int absolute)
      * the broken sender software intends.  (This totally breaks
      * in 2100 -- setting up the next crisis...) - BL
      */
-     if( atoi( start ) >= 70 )
-       strncpy(temp, "19", 2);
-     else
-       strncpy(temp, "20", 2);
-     strncat(temp, start, 2);
-     temp[4] = '\0';
-  } 
-  else{
-    return(0);
+    if(start != NULL && atoi(start) >= 70)
+     strncpy(temp, "19", 2);
+    else
+      strncpy(temp, "20", 2);
+    strncat(temp, start, 2);
+    temp[4] = '\0';
+  } else {
+    return 0;
   }
-  year = atoi( temp );
+  if(temp != NULL)
+    year = atoi(temp);
 
   /*
    *  Get the numeric hour string and convert to an integer.
    */
-  while( *s != '\0' && !isdigit((unsigned char)*s))
+  while(*s != '\0' && !isdigit((unsigned char)*s))
     s++;
-  if( *s == '\0' ){
-    hour = 0;
+  if(*s == '\0'){
+    hour    = 0;
     minutes = 0;
     seconds = 0;
-  } 
-  else{
+  } else {
     start = s;
-    while( *s != '\0' && isdigit((unsigned char)*s ))
+    while(*s != '\0' && isdigit((unsigned char)*s))
       s++;
-    if( *s != ':' || (s - start) > 2 )
+    if(*s != ':' || (s - start) > 2)
       return(0);
-    strncpy( temp, start, (int)(s - start));
-    hour = atoi(temp);
+    strncpy(temp, start, (int)(s - start));
+    if(temp != NULL)
+      hour = atoi(temp);
 
     /*
      *  Get the numeric minutes string and convert to an integer.
      */
-    while( *s != '\0' && !isdigit(( unsigned char)*s ))
+    while(*s != '\0' && !isdigit(( unsigned char)*s))
       s++;
-    if( *s == '\0' )
+    if(*s == '\0')
       return(0);
     start = s;
-    while( *s != '\0' && isdigit(( unsigned char)*s ))
+    while(*s != '\0' && isdigit(( unsigned char)*s))
       s++;
-    if( *s != ':' || (s - start) > 2 )
+    if(*s != ':' || (s - start) > 2)
       return(0);
     strncpy(temp, start, (int)(s - start));
-    minutes = atoi(temp);
+    if(temp != NULL)
+      minutes = atoi(temp);
 
     /*
      *  Get the numeric seconds string and convert to an integer.
      */
-    while( *s != '\0' && !isdigit(( unsigned char)*s ))
+    while(*s != '\0' && !isdigit(( unsigned char)*s))
       s++;
-    if( *s == '\0' )
+    if(*s == '\0')
       return(0);
     start = s;
-    while( *s != '\0' && isdigit(( unsigned char)*s ))
+    while(*s != '\0' && isdigit(( unsigned char)*s))
       s++;
     /* We don't care if we're at the end of the string here.
        Theoretically there should be a timezone specification
        but we don't care that much if it's not there. */
-    if(( s - start) > 2 )
+    if((s - start) > 2)
       return(0);
     strncpy(temp, start, (int)(s - start));
-    seconds = atoi(temp);
+    if(temp != NULL)
+      seconds = atoi(temp);
   }
 
   /*
@@ -267,14 +278,14 @@ mktime2(const char *string, int absolute)
    *  TRUE. 
    */
   month -= 3;
-  if( month < 0 ){
+  if(month < 0){
     month += 12;
     year--;
   }
-  day += ( year - 1968 ) * 1461 / 4;
-  day += (((( month * 153 ) + 2 ) / 5 ) - 672 );
-  clock2 = (time_t)(( day * 60 * 60 * 24 ) + (hour * 60 * 60) + (minutes * 60) + seconds );
-  if( !absolute && (long)(time((time_t *)0) - clock2) >= 0 )
+  day += (year - 1968) * 1461 / 4;
+  day += ((((month * 153) + 2) / 5) - 672);
+  clock2 = (time_t)((day * 60 * 60 * 24) + (hour * 60 * 60) + (minutes * 60) + seconds );
+  if(!absolute && (long)(time((time_t *)0) - clock2) >= 0)
     clock2 = (time_t)0;
   return(clock2);
 }
@@ -291,14 +302,19 @@ parse_cookie(char *cookiestr, PARSED_COOKIE* ck)
 
   lval = cookiestr;
 
-  while( *cookiestr && *cookiestr != '=' )
+  while(*cookiestr && *cookiestr != '=')
     cookiestr++;
-  if (!*cookiestr) return;
+
+  if(!*cookiestr) return;
+
   *cookiestr++ = 0; /* NULL-terminate lval (replace '=') and position at start of rval */
 
   rval = cookiestr;
-  while( *cookiestr && *cookiestr != ';' )
+  while(*cookiestr && *cookiestr != ';')
     cookiestr++;
+
+  if(!*cookiestr) return;
+
   *cookiestr++ = 0;
 
   ck->name  = (lval != NULL) ? xstrdup(lval) : NULL;
@@ -368,7 +384,7 @@ parse_cookie(char *cookiestr, PARSED_COOKIE* ck)
  * insert values into list
  */
 int
-add_cookie( pthread_t id, char *host, char *cookiestr )
+add_cookie(pthread_t id, char *host, char *cookiestr)
 {
   char *name, *value;
   int  found = FALSE;
@@ -383,7 +399,7 @@ add_cookie( pthread_t id, char *host, char *cookiestr )
   pthread_mutex_lock(&(cookie->mutex)); 
    
   for( cur=pre=cookie->first; cur != NULL; pre=cur, cur=cur->next ){
-    if(( cur->threadID == id )&&( !strcasecmp( cur->name, name ))){
+    if(( cur->threadID == id )&&( !strcasecmp(cur->name, name))){
       xfree( cur->name );
       xfree( cur->value );
       xfree( cur->domain ); 
@@ -453,7 +469,7 @@ delete_cookie( pthread_t id, char *name )
   return 0 (as delete_cookie does)?
 */
 int
-delete_all_cookies( pthread_t id )
+delete_all_cookies(pthread_t id)
 {
   CNODE  *cur, *pre;
   
@@ -487,64 +503,55 @@ delete_all_cookies( pthread_t id )
   return 0;
 }
 
-/* 
-  get_cookie returns a char * with a compete 
-  value of the Cookie: string.  It does NOT
-  return the actual Cookie struct.      
-*/
+/**
+ * get_cookie returns a char * with a compete value of the Cookie: header
+ * It does NOT return the actual Cookie struct.      
+ */
 char *
-get_cookie( pthread_t id, char *host, char *newton )
+get_cookie_header(pthread_t id, char *host, char *newton)
 {
-  char   oreo[MAX_COOKIE_SIZE];  
   int    dlen, hlen; 
   CNODE  *pre, *cur;
   time_t now;
+  char   oreo[MAX_COOKIE_SIZE];  
 
-  memset( oreo, '\0', sizeof( oreo ));
-  hlen = strlen( host ); 
-
-  /*printf ("get_cookie (%d, '%s', '%s')\n", id, host?host:"NULL", newton?newton:"NULL");*/
+  memset(oreo, '\0', sizeof oreo);
+  hlen = strlen(host); 
 
   pthread_mutex_lock(&(cookie->mutex));
   now = time(NULL);
 
-  for( cur=pre=cookie->first; cur != NULL; pre=cur, cur=cur->next ) {
-
+  for(cur=pre=cookie->first; cur != NULL; pre=cur, cur=cur->next){
     dlen = cur->domain ? strlen( cur->domain ) : 0;
-    /*
-    printf ("cur->domain='%s' dlen=%d host='%s' %s=%s\n",
-            cur->domain?cur->domain:"NULL", dlen, host?host:"NULL",
-            cur->name, cur->value?cur->value:"NULL");
-    */
 
-    if( cur->threadID == id ){
-      if( !strcasecmp( cur->domain, host )){
-        if (cur->expires <= now) {
-          /*printf ("EXPIRED COOKIE! - %s=%s\n", cur->name, cur->value?cur->value:"NULL");*/
+    if(cur->threadID == id){
+      if(!strcasecmp(cur->domain, host)){
+        if(cur->expires <= now){
           continue;
         }
-        if( strlen( oreo ) > 0 )
-          strncat( oreo, ";",      sizeof( oreo ));
-        strncat( oreo, cur->name,  sizeof( oreo ));
-        strncat( oreo, "=",        sizeof( oreo ));
-        strncat( oreo, cur->value, strlen(cur->value));
-        /*printf ("[0] APPENDED COOKIE - %s=%s\n", cur->name, cur->value?cur->value:"NULL");*/
+        if(strlen(oreo) > 0)
+          strncat(oreo, ";",      sizeof oreo);
+        strncat(oreo, cur->name,  sizeof oreo);
+        strncat(oreo, "=",        sizeof oreo);
+        strncat(oreo, cur->value, strlen(cur->value));
       }
-      if(( dlen < hlen ) && ( !strcasecmp( host + ( hlen - dlen ), cur->domain ))){
+      if((dlen < hlen) && (!strcasecmp(host + (hlen - dlen), cur->domain))){
         if (cur->expires <= now) {
-          /*printf ("EXPIRED COOKIE! - %s=%s\n", cur->name, cur->value?cur->value:"NULL");*/
           continue;
         }
-        if( strlen( oreo ) > 0 )
-          strncat( oreo, ";",      sizeof( oreo ));
-        strncat( oreo, cur->name,  sizeof( oreo ));
-        strncat( oreo, "=",        sizeof( oreo ));
-        strncat( oreo, cur->value, strlen(cur->value));
-        /*printf ("[1] APPENDED COOKIE - %s=%s\n", cur->name, cur->value?cur->value:"NULL");*/
+        if(strlen(oreo) > 0)
+          strncat(oreo, ";",      sizeof oreo);
+        strncat(oreo, cur->name,  sizeof oreo);
+        strncat(oreo, "=",        sizeof oreo);
+        strncat(oreo, cur->value, strlen(cur->value));
       }
     }
   }
-  strncpy( newton, oreo, sizeof( oreo ));
+  if(strlen(oreo) > 0){
+    strncpy(newton, "Cookie: ", 8);
+    strncat(newton, oreo, sizeof(oreo));
+    strncat(newton, "\015\012", 2);
+  }
   pthread_mutex_unlock(&(cookie->mutex));
 
   return newton;
