@@ -34,7 +34,7 @@ int  add_cookie( pthread_t id, char *host, char *value );
 int  delete_cookie( pthread_t id, char *name ); 
 int  delete_all_cookies( pthread_t id );
 int  check_cookie( char *domain, char *value );
-char *get_cookie( pthread_t id, char *domain, char *cookie );
+char *get_cookie_header(pthread_t id, char *domain, char *cookie);
 
 #endif/*COOKIES_H*/
 
