@@ -32,27 +32,28 @@
 #include <setup.h>
 #include <handler.h> 
 #include <util.h>
+#include <crew.h>
 #include <joedog/boolean.h>
 
 void
-spin_doctor()
+spin_doctor(CREW crew)
 {
   long x;
   char h[4] = {'-','\\','|','/'};
 
-  for( x = 0; our.total_threads > 1 || x < 55; x++ ){
-    fflush( stdout );
-    fprintf(stdout,"%c", h[x%4] );
-    pthread_usleep_np( 20000 );
-    fprintf( stdout, "\b" ); 
+  for(x = 0; crew_get_total(crew) > 1 || x < 55; x++){
+    fflush(stderr);
+    fprintf(stderr,"%c", h[x%4]);
+    pthread_usleep_np(20000);
+    fprintf(stderr, "\b"); 
   }
   return;
 }
 
 void 
-sig_handler( pthread_t *peer )
+sig_handler(CREW crew)
 {
-  int gotsig = 0, x; 
+  int gotsig = 0; 
   sigset_t  sigs;
 #if defined (hpux) || defined(__hpux)
 #else
@@ -70,15 +71,14 @@ sig_handler( pthread_t *peer )
    * Now wait around for something to happen ... 
    */
   sigwait(&sigs, &gotsig);
-  fprintf( stderr, "\nLifting the server siege..." ); 
-  pthread_mutex_lock( &our.mutex_lock );
-  our.shutting_down = TRUE;
-  pthread_mutex_unlock( &our.mutex_lock );
+  my.verbose = FALSE;
+  fprintf(stderr, "\nLifting the server siege..."); 
+  crew_set_shutdown(crew, TRUE);
 
 #if defined (hpux) || defined(__hpux)
 #else
-  if((result = pthread_create( &spinner, NULL, (void*)spin_doctor, NULL )) < 0){
-    joe_fatal( "failed to create handler: %d\n", result );
+  if((result = pthread_create(&spinner, NULL, (void*)spin_doctor, crew)) < 0){
+    joe_error("failed to create handler: %d\n", result);
   }   
 #endif
 
@@ -87,19 +87,15 @@ sig_handler( pthread_t *peer )
    * so we artificially extend the life of
    * the siege to make up the discrepancy. 
    */
-  pthread_usleep_np( 501125 ); 
-  
-  for( x = 0; x < our.total_clients; x ++ )
-#if defined(hpux) || defined(__hpux)
-    pthread_kill( peer[x], SIGUSR1 ); 
-#else
-    pthread_cancel( peer[x] ); 
-#endif
- 
+  pthread_usleep_np(501125); 
+
+  crew_cancel(crew);
+
 #if defined(hpux) || defined(__hpux)
 #else
-  pthread_join( spinner, NULL );
+  pthread_join(spinner, NULL);
 #endif
-  pthread_exit( NULL );
+
+  pthread_exit(NULL);
 }
 
