@@ -157,14 +157,14 @@ hash_add( HASH this, char *key, char *value )
  * in the table corresponding to key.
  */
 char *
-hash_get( HASH this, char *key )
+hash_get(HASH this, char *key)
 {
   int  x;
   NODE *node;
 
-  x = hash_genkey( this->size, key );
-  for( node = this->table[x]; node != NULL; node = node->next ){
-    if( !strcmp( node->key, key )){
+  x = hash_genkey(this->size, key);
+  for(node = this->table[x]; node != NULL; node = node->next){
+    if(!strcmp( node->key, key)){
       return(node->value);
     }
   }
@@ -177,14 +177,14 @@ hash_get( HASH this, char *key )
  * and 0 if it is not.
  */
 int
-hash_lookup( HASH this, char *key )
+hash_lookup(HASH this, char *key)
 {
   int  x;
   NODE *node;
 
-  x = hash_genkey( this->size, key );
-  for( node = this->table[x]; node != NULL; node = node->next ){
-    if( !strcmp( node->key, key )){
+  x = hash_genkey(this->size, key);
+  for(node = this->table[x]; node != NULL; node = node->next){
+    if(!strcmp(node->key, key)){
       return 1;
     }
   }
@@ -193,7 +193,7 @@ hash_lookup( HASH this, char *key )
 }
 
 char **
-hash_get_keys( HASH this )
+hash_get_keys(HASH this)
 {
   int x; 
   int i = 0;
@@ -201,11 +201,11 @@ hash_get_keys( HASH this )
   char **keys;
 
   keys = (char**)malloc(sizeof( char*) * this->entries);
-  for( x = 0; x < this->size; x ++ ){
-    for( node = this->table[x]; node != NULL; node = node->next ){
-      keys[i] = (char*)malloc( 128 );
-      memset( keys[i], 0, sizeof( keys[i] ));
-      memcpy( keys[i], (char*)node->key, strlen( node->key ));
+  for(x = 0; x < this->size; x ++){
+    for(node = this->table[x]; node != NULL; node = node->next){
+      keys[i] = (char*)malloc(128);
+      memset( keys[i], 0, sizeof(keys[i]));
+      memcpy( keys[i], (char*)node->key, strlen(node->key));
       keys[i][strlen(node->key)] = 0;
       i++;
     }
@@ -214,15 +214,15 @@ hash_get_keys( HASH this )
 }
 
 void 
-hash_free_keys( HASH this, char **keys )
+hash_free_keys(HASH this, char **keys)
 {
   int x;
-  for( x = 0; x < this->entries; x ++ )
-    if( keys[x] != NULL ){
+  for(x = 0; x < this->entries; x ++)
+    if(keys[x] != NULL){
       char *tmp = keys[x];
-      free( tmp );
+      xfree(tmp);
     }
-  free( keys );
+  xfree(keys);
 
   return;
 }
@@ -232,29 +232,29 @@ hash_free_keys( HASH this, char **keys )
  * memory which was allocated to it.
  */
 void 
-hash_destroy( HASH this )
+hash_destroy(HASH this)
 {
   int x;
   NODE *t1, *t2;
 
-  for( x = 0; x < this->size; x++ ){
+  for(x = 0; x < this->size; x++){
     t1 = this->table[x];
-    while( t1 != NULL ){
+    while(t1 != NULL){
       t2 = t1->next;
-      if( t1->key != NULL )
-        free( t1->key );
-      if( t1->value != NULL )
-        free( t1->value );
-      free( t1 );
+      if(t1->key != NULL)
+        xfree(t1->key);
+      if(t1->value != NULL)
+        xfree(t1->value);
+      xfree(t1);
       t1 = t2;      
     } 
     this->table[x] = NULL;
   }
-  if( this->table != NULL ){
-    free( this->table );
-    memset( this, 0, sizeof(HASH));
+  if(this->table != NULL){
+    xfree(this->table);
+    memset(this, 0, sizeof(HASH));
   } 
-  free(this);
+  xfree(this);
   return;
 }
 
