@@ -1,8 +1,8 @@
 /**
  * HTTP/HTTPS protocol support 
  *
- * Copyright (C) 2000, 2001, 2002 by
- * Jeffrey Fulmer - <jdfulmer@armstrong.com>
+ * Copyright (C) 2000-2005 by
+ * Jeffrey Fulmer - <jeff@joedog.org>
  * This file is distributed as part of Siege 
  *
  * This program is free software; you can redistribute it and/or modify
@@ -99,29 +99,37 @@ http_get(CONN *C, char *host, int port, char *path)
   int  rlen;
   char *protocol; 
   char *keepalive;
-  char cookie[MAX_COOKIE_SIZE];
   char authwww[128];
   char authpxy[128];
-  char request[REQBUF]; 
+  char request[REQBUF];  
+  char portstr[16];
+  char fullpath[4096];
+  char cookie[MAX_COOKIE_SIZE+10];
+
+  memset(cookie,  0, sizeof cookie);
+  memset(request, 0, sizeof request);
+  memset(portstr, 0, sizeof portstr);
 
   /* Request path based on proxy settings */
-  char fullpath[4096];
   if(my.proxy.required){
     sprintf(
-      fullpath, "%s://%s:%d%s", C->prot == 0?"http":"https", host, port, path 
+      fullpath, "%s://%s:%d%s", C->prot == HTTP?"http":"https", host, port, path 
     );
   } 
   else{
     sprintf(fullpath, "%s", path);
   }
 
-  memset(cookie,  0, sizeof(cookie));
-  memset(request, 0, sizeof(request));
+  if((port==80 && C->prot==HTTP) || (port==443 && C->prot==HTTPS)){
+    portstr[0] = '\0';  
+  } else {
+    snprintf(portstr, sizeof portstr, ":%d", port);
+  }
 
   /* HTTP protocol string */
   protocol  = (my.protocol == TRUE)?"HTTP/1.1":"HTTP/1.0";
   keepalive = (C->connection.keepalive == TRUE)?"keep-alive":"close";
-  get_cookie( pthread_self(), host, cookie ); 
+  get_cookie_header(pthread_self(), host, cookie); 
   if( C->auth.www ){
     rlen = snprintf(
       authwww, sizeof(authwww), 
@@ -143,28 +151,25 @@ http_get(CONN *C, char *host, int port, char *path)
   rlen = snprintf(
     request, sizeof( request ),
     "GET %s %s\015\012"
-    "Host: %s:%d\015\012" 
+    "Host: %s%s\015\012" 
+    "%s"
     "%s"
     "%s"
-    "Cookie: %s\015\012"
     "Accept: */*\015\012"
     "Accept-Encoding: * \015\012"
     "User-Agent: %s\015\012%s"
     "Connection: %s\015\012\015\012",  
-    fullpath, protocol, host, port,
+    fullpath, protocol, host, portstr,
     (C->auth.www==TRUE)?authwww:"",
     (C->auth.proxy==TRUE)?authpxy:"",
-    cookie, my.uagent, my.extra, keepalive 
+    (strlen(cookie) > 8)?cookie:"", 
+    my.uagent, my.extra, keepalive 
   );
   
   if(my.debug || my.get){ printf("%s\n", request); fflush(stdout); }
   if(rlen < 0 || rlen > (int)sizeof(request)) 
     joe_fatal("http_get: request buffer overrun!");
 
-  if((socket_check(C, WRITE)) == FALSE){
-    return -1;
-  } 
-
   if((socket_write(C, request, rlen)) < 0){
     return -1;
   }
@@ -183,14 +188,16 @@ http_post( CONN *C, char *host, int port, char *path, char *data, size_t len )
   char authwww[128];
   char authpxy[128]; 
   char request[REQBUF]; 
+  char portstr[16];
   char *protocol; 
   char *keepalive;
-
-  /* cookie value  */
   char cookie[MAX_COOKIE_SIZE];
-
-  /* Request path based on proxy settings */
   char fullpath[4096];
+
+  memset(cookie,  0, sizeof(cookie));
+  memset(request, 0, sizeof(request));
+  memset(portstr, 0, sizeof portstr);
+
   if( my.proxy.required ){
    sprintf(
       fullpath, 
@@ -202,13 +209,16 @@ http_post( CONN *C, char *host, int port, char *path, char *data, size_t len )
     sprintf(fullpath, "%s", path);
   }
 
-  memset(cookie,  0, sizeof(cookie));
-  memset(request, 0, sizeof(request));
+  if((port==80 && C->prot==HTTP) || (port==443 && C->prot==HTTPS)){
+    portstr[0] = '\0';  ;
+  } else {
+    snprintf(portstr, sizeof portstr, ":%d", port);
+  }
 
   /* HTTP protocol string */
-  protocol  = (my.protocol  == TRUE)?"HTTP/1.1":"HTTP/1.0";
+  protocol  = (my.protocol == TRUE)?"HTTP/1.1":"HTTP/1.0";
   keepalive = (C->connection.keepalive == TRUE)?"keep-alive":"close";
-  get_cookie(pthread_self(), host, cookie);
+  get_cookie_header(pthread_self(), host, cookie);
   if( C->auth.www ){
     rlen = snprintf(
       authwww, sizeof(authwww),
@@ -229,20 +239,21 @@ http_post( CONN *C, char *host, int port, char *path, char *data, size_t len )
   rlen = snprintf(
     request, sizeof(request),
     "POST %s %s\015\012"
-    "Host: %s:%d\015\012"
+    "Host: %s%s\015\012"
+    "%s"
     "%s"
     "%s"
-    "Cookie: %s\015\012"
     "Accept: */*\015\012"
     "Accept-Encoding: * \015\012"
     "User-Agent: %s\015\012%s"
     "Connection: %s\015\012"
     "Content-type: application/x-www-form-urlencoded\015\012"
     "Content-length: %ld\015\012\015\012",
-    fullpath, protocol, host, port,
+    fullpath, protocol, host, portstr,
     (C->auth.www==TRUE)?authwww:"",
     (C->auth.proxy==TRUE)?authpxy:"",
-    cookie, my.uagent, my.extra, keepalive, (long)len
+    (strlen(cookie) > 8)?cookie:"", 
+    my.uagent, my.extra, keepalive, (long)len
   ); 
 
   if( rlen + len < sizeof(request)){
@@ -255,11 +266,7 @@ http_post( CONN *C, char *host, int port, char *path, char *data, size_t len )
   if(rlen<0 || rlen>(int)sizeof(request) )
     joe_fatal("http_post: request buffer overrun!"); 
 
-  if((socket_check(C, WRITE)) == FALSE){
-    return -1;
-  } 
-
-  if((socket_write( C, request, rlen )) < 0){
+  if((socket_write(C, request, rlen)) < 0){
     return -1;
   }
 
@@ -274,28 +281,30 @@ http_post( CONN *C, char *host, int port, char *path, char *data, size_t len )
 HEADERS *
 http_read_headers(CONN *C, char *host)
 { 
-  int  x;           /* while loop index      */
-  int  n;           /* assign socket_read    */
-  char c;           /* assign char read      */
-  char line[1536];  /* assign chars read     */
-  HEADERS *h;       /* struct to hold it all */
+  int  x;             /* while loop index      */
+  int  n;             /* assign socket_read    */
+  char c;             /* assign char read      */
+  int  len   = 0;     /* size length of line   */
+  char *line = NULL;  /* assign chars read     */
+  HEADERS *h;         /* struct to hold it all */
   
   h = xcalloc(sizeof(HEADERS), 1);
   
-  if((socket_check(C, READ)) == FALSE){
-    return NULL;
-  } 
-
-  while( TRUE ){
+  while(TRUE){
     x = 0;
-    memset( &line, 0, sizeof( line ));
-    while(( n = socket_read( C, &c, 1 )) == 1 ){
+    memset(&line, 0, sizeof(line));
+    while((n = socket_read(C, &c, 1)) == 1){
+      if(x >= len){
+        len += 2048;
+        line = (char*)xrealloc(line, len);
+      }
       line[x] = c; 
       if(my.debug || my.get){ printf("%c", c ); fflush(stdout); }
       if((line[0] == '\n') || (line[1] == '\n')){ 
+        xfree(line);
         return h;
       }
-      if( line[x] == '\n' ) break;
+      if(line[x] == '\n') break;
       x ++;
     }
     line[x]=0;
@@ -308,93 +317,123 @@ http_read_headers(CONN *C, char *host)
     if( strncasecmp( line, "content-length: ", 16 ) == 0 ){ 
       C->content.length = atoi( line + 16 ); 
     }
-    if( strncasecmp( line, "set-cookie: ", 12 ) == 0 ){
-      if( my.cookies ){
-        memset( h->cookie, 0, sizeof( h->cookie ));
-        strncpy( h->cookie, line+12, strlen( line ));
-        add_cookie( pthread_self(), host, h->cookie );
+    if(strncasecmp( line, "set-cookie: ", 12 ) == 0){
+      if(my.cookies){
+        char* cookie = line + 12;
+        int cookie_len = strlen(cookie);
+        if(cookie_len + 1 > (int)sizeof(h->cookie)){
+          cookie_len = (int)sizeof(h->cookie) - 1;
+        }
+        memset(h->cookie, 0, sizeof(h->cookie));
+        strncpy(h->cookie, line+12, cookie_len);
+        add_cookie(pthread_self(), host, h->cookie);
       }
     }
-    if( strncasecmp( line, "connection: ", 12 ) == 0 ){
-      if( strncasecmp( line+12, "keep-alive", 10 ) == 0 ){
+    if(strncasecmp(line, "connection: ", 12) == 0){
+      if(strncasecmp(line+12, "keep-alive", 10) == 0){
         h->keepalive = 1;
       }
-      else if( strncasecmp( line+12, "close", 5 ) == 0 ){
+      else if(strncasecmp(line+12, "close", 5) == 0){
         h->keepalive = 0;
       }
     }
-    if( strncasecmp( line, "keep-alive: ", 12 ) == 0 ){
+    if(strncasecmp(line, "keep-alive: ", 12) == 0){
       char *tmp    = "";
       char *option = "", *value = "";
       char *newline = (char*)line;
-      while(( tmp = __parse_pair( &newline )) != NULL ){
+      while((tmp = __parse_pair(&newline)) != NULL){
         option = tmp;
-        while( *tmp && !ISSPACE( (int)*tmp ) && !ISSEPARATOR( *tmp ))
+        while(*tmp && !ISSPACE((int)*tmp) && !ISSEPARATOR(*tmp))
           tmp++;
         *tmp++=0;
-        while( ISSPACE( (int)*tmp ) || ISSEPARATOR( *tmp ))
+        while(ISSPACE((int)*tmp) || ISSEPARATOR(*tmp))
           tmp++;
         value  = tmp;
-        while( *tmp )
+        while(*tmp)
           tmp++;  
-        if( !strncasecmp( option, "timeout", 7 )){
-          C->connection.timeout = atoi( value );
+        if(!strncasecmp( option, "timeout", 7)){
+          C->connection.timeout = atoi(value);
         }
-        if( !strncasecmp( option, "max", 3 )){
-          C->connection.max     = atoi( value );
+        if(!strncasecmp(option, "max", 3)){
+          C->connection.max     = atoi(value);
         }
       }
     }
-    if( strncasecmp(line, "location: ", 10) == 0){
+    if(strncasecmp(line, "location: ", 10) == 0){
       size_t len  = strlen(line);
       h->redirect = xmalloc(len);
       memcpy(h->redirect, line+10, len-10);
       h->redirect[len-10] = 0;
     }
-    if( strncasecmp(line, "www-authenticate: ", 18 ) == 0 ){
+    if(strncasecmp(line, "www-authenticate: ", 18) == 0){
       char *tmp;
+      int   tmp_len;
       h->auth.www = TRUE;
-      if( strncasecmp( line+19, "digest", 6 ) == 0 ){
+      if(strncasecmp(line+19, "digest", 6) == 0){
         h->auth.type.www = DIGEST;
-      } 
-      else{
+      } else {
         h->auth.type.www = BASIC;
       }
       tmp = strchr( line, '=' );
-      tmp++;
-      if( tmp[0] == '"' ){ tmp++; tmp[strlen(tmp)-1] = '\0'; }
-      strncpy( h->auth.realm.www, tmp, strlen( tmp )); 
-    }
-    if( strncasecmp(line, "proxy-authenticate: ", 20 ) == 0 ){
-      char *tmp;
-      h->auth.proxy = TRUE;
-      if( strncasecmp( line+21, "digest", 6 ) == 0 ){
-        h->auth.type.proxy = DIGEST;
+      if(h->auth.www && tmp) {
+        tmp++;
+        tmp_len = strlen(tmp);
+        if(tmp[0] == '"' && tmp[tmp_len - 1] == '"'){ 
+          tmp++; 
+          tmp_len -= 2; 
+          tmp[tmp_len] = '\0'; 
+        }
+        if(tmp_len + 1 > (int)sizeof(h->auth.realm.www)){ 
+          tmp_len = (int)sizeof(h->auth.realm.www) - 1;
+        }
+        strncpy( h->auth.realm.www, tmp, tmp_len );
       }
-      else{
+    }
+    if(strncasecmp(line, "proxy-authenticate: ", 20) == 0){
+      char *tmp;
+      int   tmp_len;
+      h->auth.proxy = TRUE;
+      if(strncasecmp(line+21, "digest", 6) == 0){
+        h->auth.type.proxy = DIGEST;
+      } else {
         h->auth.type.proxy = BASIC;
       }   
-      tmp = strchr( line, '=' );
-      tmp++;
-      if( tmp[0] == '"' ){ tmp++; tmp[strlen(tmp)-1] = '\0'; }
-      strncpy( h->auth.realm.proxy, tmp, strlen( tmp )); 
+      tmp = strchr(line, '=');
+      if(h->auth.proxy && tmp) {
+        tmp++;
+        tmp_len = strlen(tmp);
+        if(tmp[0] == '"' && tmp[tmp_len - 1] == '"'){ 
+          tmp++; 
+          tmp_len -= 2; 
+          tmp[tmp_len] = '\0'; 
+        }
+        if(tmp_len + 1 > (int)sizeof(h->auth.realm.proxy)){
+          tmp_len = (int)sizeof(h->auth.realm.proxy) - 1;
+        }
+        strncpy(h->auth.realm.proxy, tmp, tmp_len);
+      }
     }                      
-    if( strncasecmp(line, "transfer-encoding: ", 19 ) == 0 ){
-      if( strncasecmp( line+20, "chunked", 7 )){
+    if(strncasecmp(line, "transfer-encoding: ", 19) == 0){
+      if(strncasecmp(line+20, "chunked", 7)){
         C->content.transfer = CHUNKED; 
-      } 
-      else if( strncasecmp( line+20, "trailer", 7 )){
+      } else if(strncasecmp( line+20, "trailer", 7)){
         C->content.transfer = TRAILER; 
-      } 
-      else{
+      } else {
         C->content.transfer = NONE;
       }
     }
-    if( n <  0 ){ 
-      return( NULL ); 
-    } /* socket closed */
+    if(n == 0){
+      xfree(line);
+      return h;
+    }
+    if(n <  0){
+      xfree(line);
+      xfree(h);
+      return(NULL); 
+    } 
   } /* end of while TRUE */
 
+  xfree(line);
   return h;
 }
 
@@ -402,25 +441,25 @@ int
 http_chunk_size(CONN *C)
 {
   int    n;
-  char   line[256];
   char   *end;
   size_t length;
 
-  memset( line, 0, sizeof( line ));
-  if(( n = socket_readline( C, line, sizeof( line ))) < 1 ){
-    joe_error("readline: unable to determine chunk size");
+  memset(C->chkbuf, 0, sizeof(C->chkbuf));
+  if((n = socket_readline(C, C->chkbuf, sizeof(C->chkbuf))) < 1){
+    joe_error("http.c: unable to determine chunk size");
     return -1;
   }
-  if((( line[0] == '\n' )||(strlen(line)==0)||(line[0] == '\r'))){
+
+  if(((C->chkbuf[0] == '\n')||(strlen(C->chkbuf)==0)||(C->chkbuf[0] == '\r'))){
     return -1;
   }
  
   errno  = 0;
-  if(!isxdigit(*line))
+  if(!isxdigit((unsigned)*C->chkbuf))
     return -1;
-  length = strtoul(line, &end, 16);
-  if(( errno == ERANGE ) || ( end == line )){
-    joe_error( "bad chunk line %s\n", line );
+  length = strtoul(C->chkbuf, &end, 16);
+  if((errno == ERANGE) || (end == C->chkbuf)){
+    joe_error("http.c: bad chunk line %s\n", C->chkbuf);
     return 0;
   } else {
     return length;
@@ -429,14 +468,12 @@ http_chunk_size(CONN *C)
 }
   
 /**
- * returns int
- * reads a http/https socket
- * ( you know what I mean :)
+ * returns ssize_t
  */
 ssize_t
-http_read( CONN *C )
+http_read(CONN *C)
 { 
-  int    n = 0;
+  int    n      = 0;
   int    chunk  = 0;
   size_t bytes  = 0;
   size_t length = 0;
@@ -445,7 +482,7 @@ http_read( CONN *C )
   if( C == NULL )
     joe_fatal("C is NULL!\n"); 
 
-  if( C->content.length > 0 ){
+  if(C->content.length > 0){
     length = (C->content.length < MAXFILE)?C->content.length:MAXFILE;
     do {
       memset(body, 0, sizeof(body));
@@ -453,11 +490,10 @@ http_read( CONN *C )
         break;
       bytes += n;
       length = (C->content.length - bytes < MAXFILE)?C->content.length-bytes:MAXFILE;
-    } while( bytes <= C->content.length ); 
-  }
-  else if(my.chunked && C->content.transfer == CHUNKED){
+    } while(bytes < C->content.length); 
+  } else if(my.chunked && C->content.transfer == CHUNKED) {
     int tries = 0;
-    while(tries < 4) {
+    while(tries < 256) {
       chunk = http_chunk_size(C);
       if(chunk == 0)
         break;
@@ -473,11 +509,10 @@ http_read( CONN *C )
         bytes += n;
       } while(chunk > 0);
     }
-  }
-  else{
+  } else {
     do {
       memset(body, 0, sizeof(body));
-      if(( n = socket_read(C, body, sizeof(body))) == 0 )
+      if((n = socket_read(C, body, sizeof(body))) == 0)
         break;
       bytes += n;
     } while(TRUE);
