@@ -1,8 +1,8 @@
 /**
  * Log file support
  *
- * Copyright (C) 2000, 2001, 2002 by
- * Jeffrey Fulmer - <jdfulmer@armstrong.com>
+ * Copyright (C) 2000-2005 by
+ * Jeffrey Fulmer - <jeff@joedog.org>
  * This file is distributed as part of Siege 
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,11 +23,13 @@
 #ifndef  LOG_H
 #define  LOG_H
 #include <stdlib.h>
+#include <data.h>
 
-void write_to_log( int count, float elapsed, int bytes, float ttime, int code, int failed );
-void mark_log_file( char *message ); 
-int file_exists( char *file );
-int create_logfile( const char *file );
+void    log_transaction(DATA D, int failed);
+void    write_to_log(int count, float elapsed, int bytes, float ttime, int code, int failed);
+void    mark_log_file(char *message); 
+BOOLEAN file_exists(char *file);
+BOOLEAN create_logfile(const char *file);
 
 #endif/*LOG_H*/
 
