@@ -42,6 +42,8 @@
 #include <url.h>
 #include <ssl.h>
 #include <cookie.h>
+#include <crew.h>
+#include <data.h>
 #include <sys/resource.h>
 #ifdef __CYGWIN__
 # include <getopt.h>
@@ -49,15 +51,6 @@
 # include <joedog/getopt.h>
 #endif 
 
-int fd         =    0;    /* tmp file, for user defined URL          */
-int count      =    0;    /* count messages processes, ie. transacts */
-int code       =    0;    /* count HTTP successes, i.e., <  400      */
-int fail       =    0;    /* count HTTP failures,  i.e., >= 400      */ 
-clock_t start, stop;      /* process start and stop times.           */
-char temp[32];            /* container for tmp file name.            */
-
-struct tms t_start, t_stop;
-
 /**
  * long options, std options struct
  */
@@ -80,7 +73,8 @@ static struct option long_options[] =
   { "rc",         required_argument, NULL, 'R' }, 
   { "url",        required_argument, NULL, 'u' },
   { "mark",       required_argument, NULL, 'm' },
-  { "header",     required_argument, NULL, 'H' } 
+  { "header",     required_argument, NULL, 'H' },
+  { "user-agent", required_argument, NULL, 'A' }
 };
 
 /**
@@ -90,7 +84,7 @@ static struct option long_options[] =
  * return void
  */
 void 
-display_version( int i )
+display_version(BOOLEAN b)
 {
   /**
    * version_string is defined in version.c 
@@ -98,13 +92,13 @@ display_version( int i )
    * to parse it in configure.  
    */
   if(my.debug)
-    printf("%s %s: debugging enabled\n", program_name, version_string);
+    fprintf(stderr,"%s %s: debugging enabled\n", program_name, version_string);
   else 
-    printf("%s %s\n", program_name, version_string);
+    fprintf(stderr,"%s %s\n", program_name, version_string);
   /**
    * if TRUE, exit 
    */
-  if( i == 1 ){ exit( EXIT_SUCCESS ); }
+  if(b == TRUE){ exit(EXIT_SUCCESS); }
 }  /* end of display version */
 
 /**
@@ -117,32 +111,33 @@ display_help()
   /**
    * call display_version, but do not exit 
    */
-  display_version( FALSE ); 
+  display_version(FALSE); 
   printf("Usage: %s [options]\n", program_name);
   printf("       %s [options] URL\n", program_name);
   printf("       %s -g URL\n", program_name);
   printf("Options:\n"                    );
-  puts("  -V, --version         VERSION, prints version number to screen.");
-  puts("  -h, --help            HELP, prints this section.");
-  puts("  -C, --config          CONFIGURATION, show the current configuration.");
-  puts("  -v, --verbose         VERBOSE, prints notification to screen.");
-  puts("  -g, --get             GET, pull down headers from the server and display HTTP");
-  puts("                        transaction. Great for web application debugging.");
-  puts("  -c, --concurrent=NUM  CONCURRENT users, default is 10");
-  puts("  -u, --url=\"URL\"       URL, a single user defined URL for stress testing." );
-  puts("  -i, --internet        INTERNET user simulation, hits the URLs randomly." );
-  puts("  -b, --benchmark       BENCHMARK, signifies no delay for time testing." );
-  puts("  -t, --time=NUMm       TIME based testing where \"m\" is the modifier S, M, or H" );
-  puts("                        no space between NUM and \"m\", ex: --time=1H, one hour test." );
-  puts("  -r, --reps=NUM        REPS, number of times to run the test, default is 25" );
-  puts("  -f, --file=FILE       FILE, change the configuration file to file." );
-  printf("  -R, --rc=FILE         RC, change the %src file to file.  Overrides\n",program_name);
-  puts("                        the SIEGERC environmental variable.");
-  printf("  -l, --log             LOG, logs the transaction to PREFIX/var/%s.log\n", program_name);
-  puts("  -m, --mark=\"text\"     MARK, mark the log file with a string separator." );
-  puts("  -d, --delay=NUM       Time DELAY, random delay between 1 and num designed" );
-  puts("                        to simulate human activity. Default value is 3" );
-  puts("  -H, --header=\"text\"   Add a header to request (can be many)" ); 
+  puts("  -V, --version           VERSION, prints version number to screen.");
+  puts("  -h, --help              HELP, prints this section.");
+  puts("  -C, --config            CONFIGURATION, show the current configuration.");
+  puts("  -v, --verbose           VERBOSE, prints notification to screen.");
+  puts("  -g, --get               GET, pull down headers from the server and display HTTP");
+  puts("                          transaction. Great for web application debugging.");
+  puts("  -c, --concurrent=NUM    CONCURRENT users, default is 10");
+  puts("  -u, --url=\"URL\"         URL, a single user defined URL for stress testing." );
+  puts("  -i, --internet          INTERNET user simulation, hits the URLs randomly." );
+  puts("  -b, --benchmark         BENCHMARK, signifies no delay for time testing." );
+  puts("  -t, --time=NUMm         TIME based testing where \"m\" is the modifier S, M, or H" );
+  puts("                          no space between NUM and \"m\", ex: --time=1H, one hour test." );
+  puts("  -r, --reps=NUM          REPS, number of times to run the test, default is 25" );
+  puts("  -f, --file=FILE         FILE, change the configuration file to file." );
+  printf("  -R, --rc=FILE           RC, change the %src file to file.  Overrides\n",program_name);
+  puts("                          the SIEGERC environmental variable.");
+  printf("  -l, --log               LOG, logs the transaction to PREFIX/var/%s.log\n", program_name);
+  puts("  -m, --mark=\"text\"       MARK, mark the log file with a string separator." );
+  puts("  -d, --delay=NUM         Time DELAY, random delay between 1 and num designed" );
+  puts("                          to simulate human activity. Default value is 3" );
+  puts("  -H, --header=\"text\"     Add a header to request (can be many)" ); 
+  puts("  -A, --user-agent=\"text\" Sets User-Agent in request" ); 
   /**
    * our work is done, exit nicely
    */
@@ -154,14 +149,14 @@ display_help()
  * the options are initialized from the .siegerc file before the command line
  * switches are parsed. The argument index is reset before leaving the
  * function. */
-void parse_rc_cmdline( int argc, char *argv[] )
+void 
+parse_rc_cmdline(int argc, char *argv[])
 {
   int a = 0;
   strcpy(my.rc, "");
   
-  while( a > -1 )
-  {
-    a = getopt_long(argc, argv, "VhvCDglibr:t:f:d:c:u:m:H:R:", long_options, (int*)0);
+  while( a > -1 ){
+    a = getopt_long(argc, argv, "VhvCDglibr:t:f:d:c:u:m:H:R:A:", long_options, (int*)0);
     if(a == 'R'){
       strcpy(my.rc, optarg);
       a = -1;
@@ -176,19 +171,19 @@ void parse_rc_cmdline( int argc, char *argv[] )
  * getopts included with this distribution.  
  */ 
 void 
-parse_cmdline( int argc, char *argv[] )
+parse_cmdline(int argc, char *argv[])
 {
   int c = 0;
   int nargs;
-  while(( c = getopt_long( argc, argv, "VhvCDglibr:t:f:d:c:u:m:H:R:", 
-          long_options, (int *)0)) != EOF ){
-  switch( c ){
+  while((c = getopt_long( argc, argv, "VhvCDglibr:t:f:d:c:u:m:H:R:A:", 
+          long_options, (int *)0)) != EOF){
+  switch(c){
       case 'V':
-        display_version( TRUE );
+        display_version(TRUE);
         break;
       case 'h':
         display_help();
-        exit( EXIT_SUCCESS );
+        exit(EXIT_SUCCESS);
       case 'D':
         my.debug = TRUE;
         break;
@@ -196,7 +191,7 @@ parse_cmdline( int argc, char *argv[] )
         my.config = TRUE;
         break;
       case 'c':
-        my.cusers  = atoi( optarg );
+        my.cusers  = atoi(optarg);
         break;
       case 'i':
         my.internet = TRUE;
@@ -226,19 +221,22 @@ parse_cmdline( int argc, char *argv[] )
         my.verbose = TRUE;
         break;
       case 'r':
-        my.reps = atoi( optarg );
+        my.reps = atoi(optarg);
         break;
       case 't':
-        parse_time( optarg );
+        parse_time(optarg);
         break;
       case 'f':
-        memset( my.file, 0, sizeof( my.file ));
-        if( optarg == NULL ) break; /*paranoia*/
-        strncpy( my.file, optarg, strlen( optarg ));
+        memset(my.file, 0, sizeof(my.file));
+        if(optarg == NULL) break; /*paranoia*/
+        strncpy(my.file, optarg, strlen(optarg));
         break;
       case 'u':
-        my.url = (char*)strdup( optarg ); 
+        my.url = (char*)strdup(optarg); 
         break;
+      case 'A':
+		strncpy(my.uagent, optarg, 255);
+		break;
       case 'R':  
         /**
          * processed above 
@@ -269,23 +267,19 @@ parse_cmdline( int argc, char *argv[] )
 int 
 main(int argc, char *argv[])
 {
-  int x, result;                /* index, func. result  */
-  float elapsed;                /* total elapsed time   */
-  float favail, fcount, ffailed;
-  float ttime = 0.0;	        /* total time of test   */
-  unsigned int  randrseed;      /* seed pthread_rand_np */
-  unsigned long int bytes = 0;  /* total bytes trans.   */
-  LINES            *lines;      /* defined in setup.h   */
-  CLIENT           *client;     /* defined in setup.h   */
-  URL              **urls;      /* urls struct array    */
-  pthread_t        *peer;       /* array of pthreads    */
-  pthread_t        cease;       /* the shutdown thread  */
-  pthread_t        timer;       /* the timer thread     */
-  pthread_attr_t   scope_attr;  /* set to PTHREAD_SCOPE_SYSTEM */
+  int    x; 
+  int    result;
+  DATA           D = new_data();
+  CREW           crew;           /* thread pool - crew.h */
+  LINES          *lines;         /* defined in setup.h   */
+  CLIENT         *client;        /* defined in setup.h   */
+  URL            **urls;         /* urls struct array    */
+  pthread_t      cease;          /* the shutdown thread  */
+  pthread_t      timer;          /* the timer thread     */
+  pthread_attr_t scope_attr;     /* set to PTHREAD_SCOPE_SYSTEM */
+  unsigned int   randrseed;      /* seed pthread_rand_np */
   void *statusp;
   sigset_t sigs;
-  float highest_trans = 0.0;
-  float lowest_trans = -1;
 
   sigemptyset(&sigs);
   sigaddset(&sigs, SIGHUP);
@@ -299,7 +293,6 @@ main(int argc, char *argv[])
   lines->line    = NULL;
 
   memset(&my, 0, sizeof(struct CONFIG));
-  memset(&our, 0, sizeof(struct O_STATUS)); 
 
   parse_rc_cmdline(argc, argv); 
   if(init_config() < 0){        /* defined in init.h   */
@@ -315,36 +308,47 @@ main(int argc, char *argv[])
     my.bench   = TRUE;
   } 
 
-  if(my.config)      show_config(TRUE);    
+  if(my.config){
+    show_config(TRUE);    
+  }
 
-  if(my.url != NULL) my.length = 1; 
-  else               my.length = read_cfg_file( lines, my.file ); 
+  if(my.url != NULL){
+    my.length = 1; 
+  } else { 
+    my.length = read_cfg_file( lines, my.file ); 
+  }
 
-  if( my.length == 0 ) 
+  if(my.length == 0){ 
     display_help();
+  }
 
   /* cookie is an EXTERN, defined in setup */ 
   cookie = xcalloc(sizeof(COOKIE), 1); 
   cookie->first = NULL;
-  if(( result = pthread_mutex_init( &(cookie->mutex), NULL)) !=0 ){
+  if((result = pthread_mutex_init( &(cookie->mutex), NULL)) !=0){
     joe_fatal( "pthread_mutex_init" );
   } 
 
   /* memory allocation for threads and clients */
   urls   = xmalloc(my.length * sizeof(URL));
   client = xmalloc(sizeof(CLIENT) * my.cusers);
+#if 0
   peer   = (pthread_t*)xmalloc(sizeof(pthread_t) * my.cusers);
+#endif
+  if((crew = new_crew(my.cusers, my.cusers, FALSE)) == NULL){
+    joe_fatal("unable to allocate memory for %d simulated browser", my.cusers);  
+  }
 
   /** 
    * determine the source of the url(s),
    * command line or file, and add them
    * to the urls struct.
    */
-  if(my.url != NULL)
-    urls[0]   =  add_url(my.url);       /* from command line  */
-  else{ 
+  if(my.url != NULL){
+    urls[0]   =  add_url(my.url, 1);         /* from command line  */
+  } else { 
     for(x = 0; x < my.length; x ++){
-      urls[x] =  add_url(lines->line[x]); /* from urls.txt file */
+      urls[x] =  add_url(lines->line[x], x); /* from urls.txt file */
     }
   } 
 
@@ -353,30 +357,32 @@ main(int argc, char *argv[])
    * to the user and prepare for verbose 
    * output if necessary.
    */
-  fprintf( stderr, "** " ); 
-  display_version( FALSE );
-  fprintf( stderr, "** Preparing %d concurrent users for battle.\n", my.cusers );
-  fprintf( stderr, "The server is now under siege..." );
-  if(my.verbose){ fprintf(stderr, "\n"); }
+  if(!my.get){
+    fprintf(stderr, "** "); 
+    display_version(FALSE);
+    fprintf(stderr, "** Preparing %d concurrent users for battle.\n", my.cusers);
+    fprintf(stderr, "The server is now under siege...");
+    if(my.verbose){ fprintf(stderr, "\n"); }
+  }
 
   /**
    * record start time before spawning threads
    * as the threads begin hitting the server as
    * soon as they are created.
    */
-  start = times(&t_start);
+  data_set_start(D);
 
   /**
    * for each concurrent user, spawn a thread and
    * loop until condition or pthread_cancel from the
    * handler thread.
    */
-  pthread_attr_init( &scope_attr );
-  pthread_attr_setscope( &scope_attr, PTHREAD_SCOPE_SYSTEM );
+  pthread_attr_init(&scope_attr);
+  pthread_attr_setscope(&scope_attr, PTHREAD_SCOPE_SYSTEM);
 #if defined(_AIX)
   /* AIX, for whatever reason, defies the pthreads standard and  *
    * creates threads detached by default. (see pthread.h on AIX) */
-  pthread_attr_setdetachstate( &scope_attr, PTHREAD_CREATE_JOINABLE );
+  pthread_attr_setdetachstate(&scope_attr, PTHREAD_CREATE_JOINABLE);
 #endif
 
   /** 
@@ -392,12 +398,12 @@ main(int argc, char *argv[])
    * ctrl-C (sigterm) and the timer thread sends
    * sigterm to cease on time out.
    */
-  if((result = pthread_create( &cease, NULL, (void*)sig_handler, (void*)peer )) < 0){
-    joe_fatal( "failed to create handler: %d\n", result );
+  if((result = pthread_create(&cease, NULL, (void*)sig_handler, (void*)crew)) < 0){
+    joe_fatal("failed to create handler: %d\n", result);
   }
   if(my.secs > 0){
-    if((result = pthread_create( &timer, NULL, (void*)siege_timer, (void*)cease )) < 0){
-      joe_fatal( "failed to create handler: %d\n", result );
+    if((result = pthread_create(&timer, NULL, (void*)siege_timer, (void*)cease)) < 0){
+      joe_fatal("failed to create handler: %d\n", result);
     } 
   }
   
@@ -406,7 +412,7 @@ main(int argc, char *argv[])
   /**
    * loop until my.cusers and create a corresponding thread...
    */  
-  for( x = 0; x < my.cusers && our.shutting_down != TRUE; x++ ){
+  for( x = 0; x < my.cusers && crew_get_shutdown(crew) != TRUE; x++ ){
     client[x].id    = x; 
     client[x].bytes = 0;
     client[x].time  = 0.0;
@@ -415,26 +421,17 @@ main(int argc, char *argv[])
     client[x].fail  = 0; 
     client[x].U     = urls;
     client[x].rand_r_SEED = pthread_rand_np(&randrseed);
-    result = pthread_create(&(peer[x]), &scope_attr, (void*)start_routine, &(client[x]));
-    if( result == EAGAIN ){ 
+    result = crew_add(crew, (void*)start_routine, &(client[x]));
+    if(result == FALSE){ 
       my.verbose = FALSE;
       fprintf(stderr, "Unable to spawn additional threads; you may need to\n");
       fprintf(stderr, "upgrade your libraries or tune your system in order\n"); 
-      fprintf(stderr, "to exceed %d users.\n", my.cusers );
-      joe_fatal( "system resources exhausted" ); 
+      fprintf(stderr, "to exceed %d users.\n", my.cusers);
+      joe_fatal("system resources exhausted"); 
     }
-    pthread_usleep_np( 1000 ); 
   } /* end of for pthread_create */
 
-  /**
-   * our.total_clients is the threads high water
-   * mark; join all the threads that were spawned.
-   */
-  x = 0;
-  do{
-    pthread_join( peer[x++], &statusp );
-    fflush( stdout );
-  } while( x < our.total_clients ); 
+  crew_join(crew, TRUE, &statusp);
 
 #ifdef HAVE_SSL
   SSL_thread_cleanup();
@@ -444,26 +441,36 @@ main(int argc, char *argv[])
    * collect all the data from all the threads that
    * were spawned by the run.
    */
-  for( x = 0; x < our.total_clients; x++ ){
-    count += client[x].hits;
-    bytes += client[x].bytes;
-    ttime += client[x].time;
-    code  += client[x].code;
-    fail  += client[x].fail;
+  for(x = 0; x < ((crew_get_total(crew) > my.cusers || 
+                   crew_get_total(crew)==0 ) ? my.cusers : crew_get_total(crew)); x++){
+    data_increment_count(D, client[x].hits);
+    data_increment_bytes(D, client[x].bytes);
+    data_increment_total(D, client[x].time);
+    data_increment_code (D, client[x].code);
+    data_increment_fail (D, client[x].fail);
+    data_set_highest    (D, client[x].bigtime);
+    data_set_lowest     (D, client[x].smalltime);
     client[x].rand_r_SEED = pthread_rand_np(&randrseed);
-    if (client[x].bigtime > highest_trans) {
-      highest_trans = client[x].bigtime;
-    }
-    if ((lowest_trans < 0) || (client[x].smalltime < lowest_trans)) {
-      lowest_trans = client[x].smalltime;
-    }
   } /* end of stats accumulation */
   
   /**
    * record stop time
    */
-  stop = times(&t_stop);
-  if(my.get) exit(EXIT_SUCCESS);
+  data_set_stop(D);
+
+  /**
+   * cleanup crew
+   */ 
+  crew_destroy(crew);
+
+  if(my.get){
+    if(data_get_count(D) > 0){
+      exit(EXIT_SUCCESS);
+    } else {
+      printf("[done]\n");
+      exit(EXIT_FAILURE);
+    }
+  }
 
   /**
    * take a short nap  for  cosmetic  effect
@@ -478,30 +485,37 @@ main(int argc, char *argv[])
   /**
    * prepare and print statistics.
    */
-  ffailed = my.failed;
-  fcount  = count;
-  elapsed = elapsed_time( stop - start);
-  favail  = (fcount==0)?0:((fcount/(fcount+ffailed))*100);
-  if(my.failed >= my.failures){
-    printf("%s aborted due to excessive socket failure; you\n", program_name);
-    printf("can change the failure threshold in $HOME/.%src\n", program_name);
+  if(my.failures > 0 && my.failed >= my.failures){
+    fprintf(stderr, "%s aborted due to excessive socket failure; you\n", program_name);
+    fprintf(stderr, "can change the failure threshold in $HOME/.%src\n", program_name);
   }
-  printf( "\bTransactions:\t\t%12d hits\n", count );
-  printf( "Availability:\t\t%12.2f %%\n", favail);
-  printf( "Elapsed time:\t\t%12.2f secs\n", elapsed );
-  printf( "Data transferred:\t%12ld bytes\n", bytes );
-  printf( "Response time:\t\t%12.2f secs\n", ttime / count );
-  printf( "Transaction rate:\t%12.2f trans/sec\n", count / elapsed );
-  printf( "Throughput:\t\t%12.2f bytes/sec\n", bytes / elapsed );
-  printf( "Concurrency:\t\t%12.2f\n", ttime / elapsed );
-  printf( "Successful transactions:%12d\n", code ); 
-  printf( "Failed transactions:\t%12d\n", fail+my.failed );
-  printf( "Longest transaction:\t%12.2f\n",highest_trans );
-  printf( "Shortest transaction:\t%12.2f\n",lowest_trans );
-  puts  ( " " );
+  fprintf(stderr, "\bTransactions:\t\t%12u hits\n",        data_get_count(D));
+  fprintf(stderr, "Availability:\t\t%12.2f %%\n",          data_get_count(D)==0 ? 0 :
+                                                           (double)data_get_count(D) /
+                                                           (data_get_count(D)+my.failed)
+                                                           *100
+  );
+  fprintf(stderr, "Elapsed time:\t\t%12.2f secs\n",        data_get_elapsed(D));
+  fprintf(stderr, "Data transferred:\t%12.2f MB\n",        data_get_megabytes(D)); /*%12llu*/
+  fprintf(stderr, "Response time:\t\t%12.2f secs\n",       data_get_response_time(D));
+  fprintf(stderr, "Transaction rate:\t%12.2f trans/sec\n", data_get_transaction_rate(D));
+  fprintf(stderr, "Throughput:\t\t%12.2f MB/sec\n",        data_get_throughput(D));
+  fprintf(stderr, "Concurrency:\t\t%12.2f\n",              data_get_concurrency(D));
+  fprintf(stderr, "Successful transactions:%12u\n",        data_get_code(D)); 
+  fprintf(stderr, "Failed transactions:\t%12u\n",          my.failed);
+  fprintf(stderr, "Longest transaction:\t%12.2f\n",        data_get_highest(D));
+  fprintf(stderr, "Shortest transaction:\t%12.2f\n",       data_get_lowest(D));
+  fprintf(stderr, " \n");
+  if(my.mark)    mark_log_file(my.markstr);
+  if(my.logging) log_transaction(D, my.failed);
 
-  if(my.mark)    mark_log_file( my.markstr );
-  if(my.logging) write_to_log( count, elapsed, bytes, ttime, code, (fail+my.failed));
+  data_destroy(D);
+  if(my.url == NULL){
+    for(x = 0; x < my.length; x++)
+       xfree(lines->line[x]);
+    xfree(lines->line);
+    xfree(lines);
+  }  
 
   /**
    * exit program.
