@@ -120,9 +120,10 @@ int strlen();
 #define F_DONE        4
 
 #define MAXREPS       10301062
+#define MAX_URL       4096
 
 void display_help();
-void display_version(int i);
+void display_version(BOOLEAN b);
 
 /** 
  * configuration struct;
@@ -137,6 +138,8 @@ struct CONFIG
   char    *url;         /* URL for the single hit invocation.      */
   char    logfile[128]; /* alternative user defined simbot.log     */ 
   BOOLEAN verbose;      /* boolean, verbose output to screen       */
+  BOOLEAN csv;          /* boolean, display verbose output in CSV  */
+  BOOLEAN fullurl;      /* boolean, display full url in verbose    */
   BOOLEAN display;      /* boolean, display the thread id verbose  */
   BOOLEAN config;       /* boolean, prints the configuration       */
   int     cusers;       /* default concurrent users value.         */
@@ -177,22 +180,19 @@ struct CONFIG
     pthread_mutex_t lock; 
   } proxy;
   BOOLEAN login;        /* boolean, client must login first.       */
-  char *loginurl;       /* the initial login URL, first URL hit.   */
-  int  failures;        /* number of failed attempts before abort. */
-  int  failed;          /* total number of socket failures.        */
+  char    *loginurl;    /* the initial login URL, first URL hit.   */
+  int     failures;     /* number of failed attempts before abort. */
+  int     failed;       /* total number of socket failures.        */
   BOOLEAN expire;       /* boolean, TRUE == expire cookies ea. run */
   BOOLEAN follow;       /* boolean, TRUE == follow 302             */
   BOOLEAN zero_ok;      /* boolean, TRUE == zero bytes data is OK. */ 
-  char rc[256];         /* filename of SIEGERC file                */  
-};
-
-struct O_STATUS
-{
-  int             total_clients;  /* high water thread mark   */
-  int             total_threads;  /* number of active threads */
-  pthread_cond_t  exit_cond;      /* pthread cond. variable   */
-  pthread_mutex_t mutex_lock;     /* w lock for total_threads */
-  int             shutting_down;  /* boolean: 1=TRUE, 0=FALSE */
+  char    rc[256];      /* filename of SIEGERC file                */  
+  int     ssl_timeout;  /* SSL session timeout                     */
+  char    *ssl_cert;    /* PEM certificate file for client auth    */
+  char    *ssl_key;     /* PEM private key file for client auth    */
+  char    *ssl_ciphers; /* SSL chiphers to use : delimited         */ 
+  pthread_cond_t  cond;
+  pthread_mutex_t lock;
 };
 
 /**
@@ -229,21 +229,23 @@ typedef enum
 /**
  * authorization type
  */
-typedef enum { BASIC, DIGEST } TYPE; 
+typedef enum { UNKNOWN, BASIC, DIGEST } TYPE; 
 
 /**
  * URL struct URL
  */ 
 typedef struct
 {
-  PROTOCOL protocol;       /* currently only http/https   */
-  char *hostname;          /* DNS entry or IP address     */
-  int  port;               /* tcp port num, defs: 80, 443 */
-  char *pathname;          /* path to http resource.      */
+  int       urlid;         /* ADDED BY jason, UNIQUE ID   */
+  PROTOCOL  protocol;      /* currently only http/https   */
+  char      *hostname;     /* DNS entry or IP address     */
+  int       port;          /* tcp port num, defs: 80, 443 */
+  char      *pathname;     /* path to http resource.      */
   DIRECTIVE calltype;      /* request: GET/POST/HEAD etc. */
-  size_t postlen;
-  char *postdata;
-  char *posttemp;
+  size_t    postlen;       /* length of POST data         */
+  char      *postdata;
+  char      *posttemp;
+  char      url[MAX_URL];
 } URL;
 
 /**
@@ -251,11 +253,11 @@ typedef struct
  */
 typedef struct
 {
-  int    id;
-  int    hits;
-  size_t bytes;
-  int    code;
-  int    fail;
+  int      id;
+  unsigned long  hits;
+  unsigned long  bytes;
+  unsigned int   code;
+  unsigned int   fail;
   URL    **U;
   struct {
     int  www;
@@ -310,9 +312,8 @@ typedef struct
 #endif
  
 EXTERN struct CONFIG my;
-EXTERN struct O_STATUS our;
-EXTERN char *version_string; 
-EXTERN char *program_name;
+EXTERN char   *version_string; 
+EXTERN char   *program_name;
 EXTERN COOKIE *cookie; 
 
 #endif  /* SETUP_H */
