@@ -1,8 +1,8 @@
 /**
  * SIEGE socket library
  *
- * Copyright (C) 2000, 2001, 2002 by
- * Jeffrey Fulmer - <jdfulmer@armstrong.com>
+ * Copyright (C) 2000-2006 by
+ * Jeffrey Fulmer - <jeff@joedog.org>
  * This file is distributed as part of Siege 
  *
  * This program is free software; you can redistribute it and/or modify
@@ -19,13 +19,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+
 #ifdef  HAVE_CONFIG_H
 # include <config.h>
 #endif/*HAVE_CONFIG_H*/
 
 #include <setup.h> 
- 
 #include <sock.h>
+#include <util.h>
 #include <joedog/joedog.h>
 #include <joedog/boolean.h>
 #include <joedog/defs.h>
@@ -59,9 +60,10 @@
 /** 
  * local prototypes 
  */
-private int     __socket_block(int socket, int nonblock);
+private int     __socket_block(int socket, BOOLEAN block);
 private ssize_t __socket_write(int sock, const void *vbuf, size_t len);  
 private ssize_t __ssl_socket_write(CONN *C, const void *vbuf, size_t len);
+private BOOLEAN __socket_check(CONN *C, SDSET mode);
 
 /**
  * new_socket
@@ -71,7 +73,6 @@ int
 new_socket(CONN *C, const char *hostparam, int portparam)
 {
   int conn;
-  struct timeval timeout;
   int res;
   int opt;
   int herrno;
@@ -84,7 +85,6 @@ new_socket(CONN *C, const char *hostparam, int portparam)
   int  rc;
 #endif/*_AIX*/ 
 
-  C->sock    = -1;
   C->encrypt = FALSE;
  
   /* if we are using a proxy, then we make a socket
@@ -96,17 +96,20 @@ new_socket(CONN *C, const char *hostparam, int portparam)
     snprintf(hn, sizeof(hn), "%s", hostparam);
     port = portparam;
   }
- 
+
   /* create a socket, return -1 on failure */
-  if(( C->sock = socket( AF_INET, SOCK_STREAM, 0 )) < 0 ){
+  if((C->sock = socket(AF_INET, SOCK_STREAM, 0)) < 0){
     switch( errno ){
-      case EPROTONOSUPPORT: { joe_error("unsupported protocol");      break; }
-      case EMFILE:          { joe_error("descriptor table is full");  break; }
-      case ENFILE:          { joe_error("system file table is full"); break; }
-      case EACCES:          { joe_error("permission denied");         break; }
-      case ENOBUFS:         { joe_error("insufficient buffer space"); break; }
-      default:              { joe_error("unknown socket error");      break; }
-    } socket_close( C ); return -1;
+      case EPROTONOSUPPORT: { joe_error("unsupported protocol %s:%d",  __FILE__, __LINE__); break; }
+      case EMFILE:          { joe_error("descriptor table full %s:%d", __FILE__, __LINE__); break; }
+      case ENFILE:          { joe_error("file table full %s:%d",       __FILE__, __LINE__); break; }
+      case EACCES:          { joe_error("permission denied %s:%d",     __FILE__, __LINE__); break; }
+      case ENOBUFS:         { joe_error("insufficient buffer %s:%d",   __FILE__, __LINE__); break; }
+      default:              { joe_error("unknown socket error %s:%d",  __FILE__, __LINE__); break; }
+    } socket_close(C); return -1;
+  }
+  if(fcntl(C->sock, F_SETFD, O_NDELAY) < 0){
+    joe_error("unable to set close control %s:%d", __FILE__, __LINE__);
   }
 
 #if defined(__GLIBC__)
@@ -120,12 +123,12 @@ new_socket(CONN *C, const char *hostparam, int portparam)
   }
 #elif defined(sun)
 # ifdef HAVE_GETIPNODEBYNAME
-  hp = getipnodebyname( hn, AF_INET, 0, &herrno );
+  hp = getipnodebyname(hn, AF_INET, 0, &herrno);
 # else /* default use gethostbyname_r*/
   {
     struct hostent hent; 
     char hbf[9000]; 
-    hp = gethostbyname_r( hn, &hent, hbf, sizeof(hbf), &herrno ); 
+    hp = gethostbyname_r(hn, &hent, hbf, sizeof(hbf), &herrno); 
   }
 # endif/*HAVE_GETIPNODEBYNAME*/
 #elif defined(_AIX)
@@ -134,7 +137,7 @@ new_socket(CONN *C, const char *hostparam, int portparam)
                        (struct hostent_data *)(aixbuf + sizeof(struct hostent)));
   hp = (struct hostent*)aixbuf;
 #elif ( defined(hpux) || defined(__hpux) || defined(__osf__) )
-  hp = gethostbyname( hn );
+  hp = gethostbyname(hn);
   herrno = h_errno;
 #else
   /* simply hoping that gethostbyname is thread-safe */
@@ -143,31 +146,31 @@ new_socket(CONN *C, const char *hostparam, int portparam)
 #endif/*OS SPECIFICS*/ 
 
   if( hp == NULL ){ return -1; } 
-  memset((void*) &cli, 0, sizeof( cli ));
-  memcpy( &cli.sin_addr, hp->h_addr, hp->h_length );
+  memset((void*) &cli, 0, sizeof(cli));
+  memcpy(&cli.sin_addr, hp->h_addr, hp->h_length);
 #if defined(sun)
 # ifdef  HAVE_FREEHOSTENT
   freehostent(hp);
 # endif/*HAVE_FREEHOSTENT*/ 
 #endif
   cli.sin_family = AF_INET;
-  cli.sin_port = htons( port );
+  cli.sin_port = htons(port);
 
-  if( C->connection.keepalive ){
+  if(C->connection.keepalive){
     opt = 1; 
     if(setsockopt(C->sock,SOL_SOCKET,SO_KEEPALIVE,(char *)&opt,sizeof(opt))<0){
       switch( errno ){
-        case EBADF:         { joe_error("invalid descriptor");       break; }
-        case ENOTSOCK:      { joe_error("not a socket");             break; }
-        case ENOPROTOOPT:   { joe_error("not a protocol option");    break; }
-        case EFAULT:        { joe_error("setsockopt unknown error"); break; }
-        default:            { joe_error("unknown sockopt error");    break; }
-      } socket_close( C ); return -1;
+        case EBADF:       { joe_error("invalid descriptor %s:%d",    __FILE__, __LINE__); break; }
+        case ENOTSOCK:    { joe_error("not a socket %s:%d",          __FILE__, __LINE__); break; }
+        case ENOPROTOOPT: { joe_error("not a protocol option %s:%d", __FILE__, __LINE__); break; }
+        case EFAULT:      { joe_error("setsockopt unknown %s:%d",    __FILE__, __LINE__); break; }
+        default:          { joe_error("unknown sockopt error %s:%d", __FILE__, __LINE__); break; }
+      } socket_close(C); return -1;
     }
   }
 
-  if(( __socket_block(C->sock, TRUE)) < 0 ){
-    joe_error( "socket: unable to set socket to non-blocking." );
+  if((__socket_block(C->sock, FALSE)) < 0){
+    joe_error("socket: unable to set socket to non-blocking %s:%d", __FILE__, __LINE__);
     return -1; 
   }
 
@@ -176,121 +179,99 @@ new_socket(CONN *C, const char *hostparam, int portparam)
    * evaluate the server response and check for
    * readability/writeability of the socket....
    */ 
-  conn = connect( C->sock, (struct sockaddr *)&cli, sizeof(struct sockaddr_in));
-  if( conn < 0 && errno != EINPROGRESS ){
-    switch( errno ){
-      case EACCES:        {joe_error("socket: EACCES");                  break;}
-      case EADDRNOTAVAIL: {joe_error("socket: address is unavailable."); break;}
-      case ETIMEDOUT:     {joe_error("socket: connection timed out.");   break;}
-      case ECONNREFUSED:  {joe_error("socket: connection refused.");     break;}
-      case ENETUNREACH:   {joe_error("socket: network is unreachable."); break;}
-      default:            {joe_error("socket: unknown network error." ); break;}
-    } socket_close( C ); return -1;
-  }
-  else{
-    /** 
-     * we have a connection, set the
-     * socket to non-blocking and test
-     * its integrity.
-     */
-    FD_ZERO(&C->rset);
-    FD_ZERO(&C->wset);
-    FD_SET( C->sock, &C->rset );    
-    FD_SET( C->sock, &C->wset );    
+  conn = connect(C->sock, (struct sockaddr *)&cli, sizeof(struct sockaddr_in));
+  if(conn < 0 && errno != EINPROGRESS){
+    switch(errno){
+      case EACCES:        {joe_error("socket: %d EACCES",                  pthread_self()); break;}
+      case EADDRNOTAVAIL: {joe_error("socket: %d address is unavailable.", pthread_self()); break;}
+      case ETIMEDOUT:     {joe_error("socket: %d connection timed out.",   pthread_self()); break;}
+      case ECONNREFUSED:  {joe_error("socket: %d connection refused.",     pthread_self()); break;}
+      case ENETUNREACH:   {joe_error("socket: %d network is unreachable.", pthread_self()); break;}
+      case EISCONN:       {joe_error("socket: %d already connected.",      pthread_self()); break;}
+      default:            {joe_error("socket: %d unknown network error.",  pthread_self()); break;}
+    } socket_close(C); return -1;
+  } else {
+    struct timeval timeout;
+    fd_set rs;
+    fd_set ws; 
+    FD_ZERO(&rs);
+    FD_ZERO(&ws);
+    FD_SET(C->sock, &rs);
+    FD_SET(C->sock, &ws);
     memset((void *)&timeout, '\0', sizeof( struct timeval ));
-    /**
-     * the default timeout is set in init.c, it's
-     * value can be changed by the user in .siegerc,
-     * but we'll still use a ternary condition since
-     * you can't be too safe....
-     */ 
     timeout.tv_sec  = (my.timeout > 0)?my.timeout:30;
     timeout.tv_usec = 0;
-
-    if(( res = select( C->sock+1, &C->rset, &C->wset, NULL, &timeout )) < 1 ){
-      joe_error( "socket: connection timed out." );
-      close( C->sock );
+    res = select(C->sock+1, &rs, &ws, NULL, &timeout);
+    if(res < 1){
+      fprintf(stderr, "socket: connection timed out\n");
+      socket_close(C);
       return -1; 
-    }
-    else{ 
+    } else { 
+      /**
+       * If we reconnect and receive EISCONN, then we have a successful connection
+       */
+      res = connect(C->sock, (struct sockaddr *)&cli, sizeof(struct sockaddr_in)); 
+      if((res < 0)&&(errno != EISCONN)){
+        joe_error("socket: unable to connect %s:%d", __FILE__, __LINE__);
+        socket_close(C);
+        return -1; 
+      }
       C->status = S_READING; 
     }
   } /* end of connect conditional */
 
-  /**
-   * make the socket blocking again.
-   */
-  if(( __socket_block( C->sock, FALSE )) < 0 ){
-    joe_error( "socket: unable to set socket to non-blocking." );
-    return -1;
-  } 
- 
+  if((__socket_block(C->sock, TRUE)) < 0){
+    joe_error("socket: unable to set socket to non-blocking %s:%d", __FILE__, __LINE__);
+    return -1; 
+  }
+
   C->connection.status = 1; 
-  return( C->sock );
+  return(C->sock);
 }
 
-/**
- * makes the socket non-blocking,
- * calls select with timeout and
- * returns the socket to blocking.
- */
-BOOLEAN
-socket_check(CONN *C, SDSET test)
+private BOOLEAN 
+__socket_check(CONN *C, SDSET mode)
 {
-  int res;
+  int    res;
+  fd_set fds;
+  fd_set *rs = NULL; 
+  fd_set *ws = NULL;
+  double timo;
   struct timeval timeout;
 
-  if( C->connection.tested >= RDWR ){
-    return TRUE; /* already tested */
+  FD_ZERO(&fds);
+  FD_SET (C->sock, &fds);
+  if(mode==WRITE){
+    *(&ws) = &fds;
+  } else {
+    *(&rs) = &fds;
   }
 
-  C->connection.tested |= test;  
+  timo = (my.timeout)?my.timeout:15;
+  timeout.tv_sec  = (long)timo;
+  timeout.tv_usec = 1000000L * (timo - (long)timo);
 
-  memset((void *)&timeout, '\0', sizeof( struct timeval ));
+  if(mode==WRITE){
+    __socket_block(C->sock, FALSE);
+  }
 
-  if(( __socket_block( C->sock, TRUE )) < 0 ){
-    return FALSE;
+  do {
+    res = select(C->sock + 1, rs, ws, NULL, &timeout);
+  } while(res < 0 && errno == EINTR);
+
+  if(mode==WRITE){
+    __socket_block(C->sock, TRUE);
   }
- 
-  timeout.tv_sec  = (my.timeout > 0)?my.timeout:15;
-  timeout.tv_usec = 0;
- 
-  switch( test ){
-  case READ:
-    FD_ZERO(&C->rset);
-    FD_SET( C->sock, &C->rset );
-    if(( res = select( C->sock+1, &C->rset, NULL, NULL, &timeout )) < 1 ){
-      close( C->sock );
-      return FALSE;
-    }
-    break;
-  case WRITE:
-    FD_ZERO(&C->wset);
-    FD_SET( C->sock, &C->wset );
-    if(( res = select( C->sock+1, NULL, &C->wset, NULL, &timeout )) < 1 ){
-      close( C->sock );
-      return FALSE;
-    }
-    break;
-  case RDWR:
-    FD_ZERO(&C->rset);
-    FD_ZERO(&C->wset);
-    FD_SET( C->sock, &C->rset );
-    FD_SET( C->sock, &C->wset );
-    if(( res = select( C->sock+1, &C->rset, &C->wset, NULL, &timeout )) < 1 ){
-      close( C->sock );
-      return FALSE;
-    }
-    break;
+
+  if(res == 0){
+    errno = ETIMEDOUT;
   }
- 
-  if(( __socket_block( C->sock, FALSE )) < 0 ){
-    joe_error( "socket: unable to set socket to non-blocking." );
-    return FALSE;
+
+  if(res < 1){
+    joe_warning("socket: %d select timed out", pthread_self());
   }
-  FD_CLR( C->sock, &C->rset );
-  FD_CLR( C->sock, &C->wset ); 
-  return TRUE;
+
+  return (res <= 0) ? FALSE : TRUE;
 }
 
 /**
@@ -298,37 +279,50 @@ socket_check(CONN *C, SDSET test)
  * set socket to non-blocking
  */
 private int
-__socket_block(int sock, int nonblock)
+__socket_block(int sock, BOOLEAN block)
 {
 #if HAVE_FCNTL_H 
   int flags;
   int retval;
+#elif defined(FIONBIO)
+  ioctl_t status;
+#else 
+  return sock;
+#endif
 
-  if(( flags = fcntl( sock, F_GETFL, 0 )) < 0 ){
-    switch( errno ){
-      case EACCES:          { joe_error( "fcntl: EACCES");                  break; }
-      case EBADF:           { joe_error( "fcntl: bad file descriptor");     break; }
-      case EAGAIN:          { joe_error( "fcntl: address is unavailable."); break; }
-      default:              { joe_error( "fcntl: unknown network error." ); break; }
+  if(sock==-1){
+    return sock;
+  }
+
+#if HAVE_FCNTL_H 
+  if((flags = fcntl(sock, F_GETFL, 0)) < 0){
+    switch(errno){
+      case EACCES: { joe_error("EACCES %s:%d",                 __FILE__, __LINE__); break; }
+      case EBADF:  { joe_error("bad file descriptor %s:%d",    __FILE__, __LINE__); break; }
+      case EAGAIN: { joe_error("address is unavailable %s:%d", __FILE__, __LINE__); break; }
+      default:     { joe_error("unknown network error %s:%d",  __FILE__, __LINE__); break; }
     } return -1;
   }
-  if( nonblock ){ 
-    flags |=  O_NDELAY;
-  }
-  else{
+
+  if(block){ 
     flags &= ~O_NDELAY;
+  } else {
+    flags |=  O_NDELAY;
+    #if (defined(hpux) || defined(__hpux) || defined(__osf__))
+    #else
+    flags |=  O_NONBLOCK;
+    #endif
   }
-  if(( retval = fcntl( sock, F_SETFL, flags)) < 0 ){
-    joe_error("fcntl: unable to set fcntl flags\n");
+
+  if((retval = fcntl(sock, F_SETFL, flags)) < 0){
+    joe_error("unable to set fcntl flags %s:%d", __FILE__, __LINE__);
     return -1;
   } 
   return retval;
 
-#elif defined( FIONBIO )
-  ioctl_t status;
- 
-  status = nb ? 1 : 0;
-  return ioctl( sd, FIONBIO, &status );
+#elif defined(FIONBIO)
+  status = block ? 0 : 1;
+  return ioctl(sock, FIONBIO, &status);
 #endif
 }  
 
@@ -345,17 +339,18 @@ __socket_write( int sock, const void *vbuf, size_t len )
  
   buf = vbuf;
   n   = len;
-  while( n > 0 ){
-    if(( w = write( sock, buf, n )) <= 0 ){
-      if( errno == EINTR )
+  while(n > 0){
+    if((w = write( sock, buf, n)) <= 0){
+      if(errno == EINTR){
         w = 0;
-      else
-        return( -1 );
+      } else {
+        return -1;
+      }
     }
     n   -= w;
     buf += w;
   }
-  return( len );
+  return len;
 }
 
 /**
@@ -373,91 +368,95 @@ __ssl_socket_write( CONN *C, const void *vbuf, size_t len )
   buf = vbuf;
   n   = len;
 #ifdef HAVE_SSL
-  while( n > 0 ){
-    if(( w = SSL_write( C->ssl, buf, n )) <= 0 ){
-      if( errno == EINTR )
+  while(n > 0){
+    if((w = SSL_write(C->ssl, buf, n)) <= 0){
+      if(errno == EINTR){
         w = 0;
-      else
-        return( -1 );
+      } else {
+        return -1;
+      }
     }
     n   -= w;
     buf += w;
   }
-  return( len );
+  return len;
 #else
-  w = C->sock; /* silenct the compiler */
+  w = C->sock; /* silence the compiler */
   joe_error( "protocol not supported" );
   return -1;
 #endif/*HAVE_SSL*/
 }
 
 ssize_t
-socket_read( CONN *C, void *vbuf, size_t len )
+socket_read(CONN *C, void *vbuf, size_t len)
 {
   int type;
   size_t      n;
   ssize_t     r;
   char *buf;
-  
-  pthread_setcanceltype( PTHREAD_CANCEL_DEFERRED, &type ); 
-  
+  int ret_eof = 0;
+ 
+  pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &type);
+  if(__socket_check(C, READ) == FALSE){
+    return -1;
+  }
+ 
   buf = vbuf;
   n   = len;
   if(C->encrypt == TRUE){
   #ifdef HAVE_SSL
-    while( n > 0 ){ 
-      if(( r = SSL_read( C->ssl, buf, n )) < 0 ){
+    while(n > 0){
+      if((r = SSL_read(C->ssl, buf, n)) < 0){
         if( errno == EINTR )
           r = 0;
         else
-          return( -1 );
+          return -1;
       }
-      else if( r == 0 ) break; 
+      else if(r == 0) break;
       n   -= r;
       buf += r;
     }   /* end of while    */
   #endif/*HAVE_SSL*/
-  }  
-  else{
-    while( n > 0 ){
-      if( C->inbuffer <  n ){
+  } else { 
+    while(n > 0){
+      if(C->inbuffer <  n){
         int lidos;
         memmove(C->buffer,&C->buffer[C->pos_ini],C->inbuffer);
         C->pos_ini = 0;
-        lidos = read( C->sock, &C->buffer[C->inbuffer], sizeof(C->buffer)-C->inbuffer );
-        if( lidos < 0 ){
+        lidos = read(C->sock, &C->buffer[C->inbuffer], sizeof(C->buffer)-C->inbuffer);
+        if(lidos == 0)
+          ret_eof = 1;
+        if(lidos < 0){
           if(errno==EINTR || errno==EAGAIN)
             lidos = 0;
-          if( errno==EPIPE ){
+          if(errno==EPIPE){
             return 0;
-          }
-          else{
-            joe_error( "socket: read error %s:", strerror( errno ));
+          } else {
+            joe_error("socket: read error %s %s:%d", strerror(errno), __FILE__, __LINE__);
             return 0; /* was return -1 */
           }
         }
         C->inbuffer += lidos;
       }
-      if( C->inbuffer >= n ){
+      if(C->inbuffer >= n){
         r = n;
-      }
-      else{
+      } else {
         r = C->inbuffer;
       }
-      if( r == 0 ) break; 
+      if(r == 0) break;
       memmove(buf,&C->buffer[C->pos_ini],r);
       C->pos_ini  += r;
       C->inbuffer -= r;
       n   -= r;
       buf += r;
-    } /* end of while */ 
+      if(ret_eof) break;
+    } /* end of while */
   }   /* end of else  */
-
+ 
   pthread_setcanceltype(type,NULL);
-  pthread_testcancel(); 
-  return( len - n );  
-}  
-
+  pthread_testcancel();
+  return(len - n);
+}                                                                                                    
 /**
  * this function is used for chunked
  * encoding transfers to acquire the 
@@ -471,21 +470,19 @@ socket_readline(CONN *C, char *ptr, size_t maxlen)
   char c;
 
   len = maxlen;
-  pthread_setcanceltype( PTHREAD_CANCEL_DEFERRED, &type ); 
+  pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &type); 
 
-  for( n = 1; n < len; n ++ ){
-    if(( res = socket_read(C, &c, 1)) == 1 ){
+  for(n = 1; n < len; n ++){
+    if((res = socket_read(C, &c, 1)) == 1){
       *ptr++ = c;
-      if(( c == '\n')||( c == '\r')) {
-        break;
-      }
+      if(c=='\n') break;
     }
-    else if( res == 0 ){
-      if( n == 1 ) return ( 0 ); 
+    else if(res == 0){
+      if(n == 1) 
+        return 0; 
       else 
         break; 
-    }
-    else{
+    } else {
       return -1; /* something bad happened */
     }
   } /* end of for loop */
@@ -495,7 +492,7 @@ socket_readline(CONN *C, char *ptr, size_t maxlen)
   pthread_setcanceltype(type,NULL);
   pthread_testcancel(); 
 
-  return( n );
+  return n;
 }
 
 /**
@@ -503,29 +500,32 @@ socket_readline(CONN *C, char *ptr, size_t maxlen)
  * socket_write wrapper function.
  */
 int
-socket_write( CONN *C, const void *buf, size_t len )
+socket_write(CONN *C, const void *buf, size_t len)
 {
   int     type;
   size_t bytes;
 
-  pthread_setcanceltype( PTHREAD_CANCEL_DEFERRED, &type ); 
-
+  pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &type); 
+#if 0
+  if(__socket_check(C, READ) == WRITE){
+    return -1;
+  }
+#endif
   if(C->encrypt == TRUE){
     /* handle HTTPS protocol */
     #ifdef HAVE_SSL
-    if((bytes = __ssl_socket_write( C, buf, len )) != len){
-      joe_error( "ssl_socket_write: ERROR" );
+    if((bytes = __ssl_socket_write(C, buf, len)) != len){
+      joe_error("ssl_socket_write: %s:%d", __FILE__, __LINE__);
       return -1;
     }
     #else
-      joe_error( "ssl_socket_write: protocol NOT supported" );
-      return -1;
+    joe_error("%s:%d protocol NOT supported", __FILE__, __LINE__);
+    return -1;
     #endif/*HAVE_SSL*/
-  }
-  else{
+  } else {
     /* assume HTTP */
-    if((bytes = __socket_write( C->sock, buf, len )) != len){
-      joe_error( "__socket_write" );
+    if((bytes = __socket_write(C->sock, buf, len)) != len){
+      joe_error("unable to write to socket %s:%d", __FILE__, __LINE__);
       return -1;
     }
   }
@@ -544,15 +544,15 @@ socket_write( CONN *C, const void *buf, size_t len )
 void
 socket_close(CONN *C)
 {
-  int type;
+  int   type;
+  int   ret   = 0;
 #ifdef  HAVE_SSL
-  int ret   = 0;
   int tries = 0;
 #endif/*HAVE_SSL*/
-  
+
   if(C==NULL) return;
 
-  /* HELP PLEASE: Is this necessary? */ 
+  /* XXX Is this necessary? */ 
   pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &type); 
 
   if(C->encrypt == TRUE){
@@ -570,18 +570,26 @@ socket_close(CONN *C)
       SSL_CTX_free(C->ctx);
       C->ctx = NULL;
       close(C->sock);
-      C->connection.status = 0;
-      C->connection.max    = 0;
-      C->connection.tested = 0;
+      C->sock              = -1;
+      C->connection.status =  0;
+      C->connection.max    =  0;
+      C->connection.tested =  0;
     }
 #endif/*HAVE_SSL*/
-  }
-  else {
+  } else {
     if(C->connection.reuse == 0 || C->connection.max == 1){
-      close(C->sock);
-      C->connection.status    = 0;
-      C->connection.max       = 0;
-      C->connection.tested    = 0;
+      if(C->sock != -1){
+        if((__socket_block(C->sock, FALSE)) < 0)
+          joe_error("unable to set to non-blocking %s:%d", __FILE__, __LINE__);
+        if((C->connection.status > 1)&&(ret = shutdown(C->sock, 2)) < 0)
+          joe_error("unable to shutdown the socket %s:%d", __FILE__, __LINE__);
+        if((ret = close(C->sock)) < 0)
+          joe_error("unable to close the socket %s:%d",    __FILE__, __LINE__);
+      }
+      C->sock                 = -1;
+      C->connection.status    =  0;
+      C->connection.max       =  0;
+      C->connection.tested    =  0;
     }
   }
   C = NULL;
