@@ -110,8 +110,9 @@ typedef struct
   size_t   inbuffer;
   int      pos_ini;
   char     buffer[4096];
-  fd_set   rset;
-  fd_set   wset;
+  char     chkbuf[1024];
+  fd_set   *ws;
+  fd_set   *rs;
 } CONN; 
 
 int       new_socket     (CONN *conn, const char *hostname, int port);
