@@ -4,8 +4,8 @@
  * used by configure to dynamically assign those values 
  * to documentation files.
  */
-char *version_string = "2.61";
-char *program_name   = "siege";
-char *author_name    = "Jeffrey Fulmer";
-char *email_address  = "jeff@joedog.org";
+const char *version_string = "2.65";
+const char *program_name   = "siege";
+const char *author_name    = "Jeffrey Fulmer";
+const char *email_address  = "jeff@joedog.org";
 
